from collections import defaultdict

from common import ScriptError, remove_readonly_fields

from ckanclient import CkanApiError

class OfstedFix:
    '''Fixes old ONS imports misattributed to Ofsted DGU#780'''
    def __init__(self, ckanclient, dry_run):
        self.ckanclient = ckanclient
        self.dry_run = dry_run

    def run(self):
        limit = 10
        def search(page=None):
            opts = {'limit': limit}
            if page != None:
                opts['offset'] = page * limit
            return self.ckanclient.package_search('Education',
#                'Source agency: Education',
                opts)
        res = search()
        print 'Found %i packages possibly related.' % res['count']
        pkg_refs = res['results']
        pkgs_done = []
        pkgs_rejected = defaultdict(list) # reason: [pkgs]
        for page in range(res['count'] / limit):
            res = search(page)
            for pkg_ref in pkg_refs:
                pkg = self.ckanclient.package_entity_get(pkg_ref)
                if 'ONS' not in pkg['extras'].get('import_source', ''):
                    pkgs_rejected['Not imported from ONS'].append(pkg)
                    continue
                if pkg.get('state', 'active') != 'active':
                    pkgs_rejected['Package state = %r' % pkg.get('state')].append(pkg)
                    continue
                if 'Source agency: Education' not in pkg['notes']:
                    source_agency = '\n'.join([line for line in pkg['notes'].split('\n') if 'Source agency' in line])
                    pkgs_rejected['Source agency = %r' % source_agency].append(pkg)
                    continue

                pkg_name = pkg['name']
                dept = pkg['extras'].get('department')
                agency = pkg['extras'].get('agency')
                author = pkg['author']
                print '%r :\n %r %r %r' % (pkg_name, dept, agency, author)
                if not self.dry_run:
                    pkg['extras']['department'] = 'Department for Education'
                    pkg['extras']['agency'] = ''
                    pkg['author'] = 'Department for Education'
                    remove_readonly_fields(pkg)
                    self.ckanclient.package_entity_put(pkg)
                    print '...done'
                pkgs_done.append(pkg)
        print 'Processed %i packages' % len(pkgs_done)
        print 'Rejected packages:'
        for reason, pkgs in pkgs_rejected.items():
            print '  %i: %s' % (len(pkgs), reason)



import os
import datetime

from sqlalchemy.util import OrderedDict
from pylons import config

from ckan.tests import *
import ckan.model as model
from ckan.lib import spreadsheet_importer
from ckanext.dgu.bis.bis import BisImporter
from ckanext.dgu.tests import PackageDictUtil
from ckan.tests.wsgi_ckanclient import WsgiCkanClient

TEST_DIR = os.path.dirname(os.path.abspath(__file__))
SAMPLES_DIR = os.path.join(TEST_DIR, 'samples')
BIS_1_FILEBASE = os.path.abspath(os.path.join(config['here'], SAMPLES_DIR, 'bis1'))
XL_EXTENSION = '.xls'

class BasicLogger:
    def __init__(self):
        self._log = []
        
    def log(self, msg):
        self._log.append(msg)

    def get_log(self):
        return self._log

def get_example_data():
    logger = BasicLogger()
    filepath = BIS_1_FILEBASE + XL_EXTENSION
    return spreadsheet_importer.XlData(logger.log, filepath=filepath, sheet_index=0)

def get_resource_data():
    logger = BasicLogger()
    filepath = BIS_1_FILEBASE + XL_EXTENSION
    return spreadsheet_importer.XlData(logger.log, filepath=filepath, sheet_index=1)

class TestSpreadsheetDataRecords:
    @classmethod
    def setup_class(self):
        self.data = get_example_data()
        self.data_records = spreadsheet_importer.SpreadsheetDataRecords(self.data, 'Dataset Ref#')
        self.records = [record for record in self.data_records.records]
        
    def test_0_title_row(self):
        assert self.data_records.titles[0] == self.data.get_row(0)[0], \
               self.data_records.titles

    def test_1_titles(self):
        titles = set(self.data_records.titles)
        expected_titles = set((
            'Dataset Ref#', 'Dataset Status', 'Agency', 'Primary Contact',
            'Secondary Contact', 'Title', 'Abstract', 'Date Released',
            'Date Updated', 'Update Frequency', 'Tags', 'Department',
            'Wiki', 'Identifier', 'Licence', 'Version', 'Geographic Coverage',
            'Geographic Granularity', 'Temporal Granularity', 'Agency',
            'Precision', 'Taxonomy url', 'Temporal Coverage From',
            'Temporal Coverage To', 'National Statistic', 'Categories',
            'Contact information', 'Data File', 'Reference Material',
            'Information', 'Full Description', 'Unknown', 'Total'))
        title_difference = expected_titles - titles
        print self.data_records.titles
        assert not title_difference, title_difference

    def test_2_records(self):
        assert len(self.records) == 2, self.records
        expected_record = [
            (u'Dataset Ref#', u'BIS-000002'),
            (u'Dataset Status', u'Proposed'),
            (u'Agency', u'Higher Education Statistics Agency'),
            (u'Primary Contact', u'information.provision@hesa.ac.uk'),
            (u'Secondary Contact', None),
            (u'Title', u'Higher Education Statistics: All HE students by level of study, mode of study, subject of study, domicile and gender 2007/08'),
            (u'Abstract', u'This dataset provides the 2007/08 higher education statistics for all students by level of study, mode of study, subject of study, domicile and gender'),
            (u'Date Released', 2008),
            (u'Date Updated', 2008),
            (u'Update Frequency', u'Never'),
            (u'Tags', u'hesa higher-education-statistics 2007-2008'),
            (u'Department', u'Department for Business, Innovation & Skills'),
            (u'Wiki', u'-'),
            (u'Identifier', u'higher-education-statistics-all-he-students-by-level-of-study-mode-of-study-subject-of-study-domicile-and-gender-2007-2008'),
            (u'Licence', u'Higher Education Statistcs Agency Copyright with data.gov.uk rights'),
            (u'Version', u'-'),
            (u'Geographic Coverage', u'United Kingdom (England, Scotland, Wales, Northern Ireland)'),
            (u'Geographic Granularity', u'national'),
            (u'Temporal Granularity', u'years'),
            (u'Precision', u'integer to the nearest 5'),
            (u'Taxonomy url', u'-'),
            (u'Temporal Coverage From', datetime.date(2007, 8, 1)),
            (u'Temporal Coverage To', datetime.date(2008, 7, 31)),
            (u'National Statistic', u'no'),
            (u'Categories', u'-'),
            (u'Contact information', u'Higher Education Statistics Agency (HESA)\n+44 (0) 1242 211133\ninformation.provision@hesa.ac.uk'),
            (u'Data File', 1),
            (u'Reference Material', 2),
            (u'Information', 0),
            (u'Full Description', 0),
            (u'Unknown', 0),
            (u'Total', 3)
            ]
        for key, value in expected_record:
            assert self.records[0][key] == value, self.records[0][key].items()
        expected_keys = set([key for key, value in expected_record])
        keys = set(self.records[0].keys())
        key_difference = expected_keys - keys
        assert not key_difference, key_difference

class TestResourceRecords:
    @classmethod
    def setup_class(self):
        self.data = get_resource_data()
        self.data_records = spreadsheet_importer.SpreadsheetDataRecords(self.data, 'Resource Status')
        self.records = [record for record in self.data_records.records]
        
    def test_0_title_row(self):
        assert self.data_records.titles[0] == self.data.get_row(0)[0], \
               self.data_records.titles

    def test_1_titles(self):
        titles = set(self.data_records.titles)
        expected_titles = set((
            'Dataset Ref#', 'Resource Status', 'Resourse Type',
            'Resource Format', 'Resource Title', 'Resource url', 'Comments',
            'Updated', 'Licence', 'Size'
            ))
        title_difference = expected_titles - titles
        print self.data_records.titles
        assert not title_difference, title_difference

    def test_2_records(self):
        assert len(self.records) == 9, self.records
        print self.records[0].items()
        expected_record = [
            (u'Dataset Ref#', u'BIS-000001'),
            (u'Resource Status', u'Proposed'),
            (u'Resourse Type', u'Data File'),
            (u'Resource Format', u'XLS'),
            (u'Resource Title', u'Data File - XLS Format'),
            (u'Resource url', u'http://www.hesa.ac.uk/dox/dataTables/studentsAndQualifiers/download/subject0809.xls?v=1.0'),
            (u'Comments', None),
            (u'Updated', None),
            (u'Licence', None),
            (u'Size', None),
            ]
        for key, value in expected_record:
            assert self.records[0][key] == value, self.records[0][key]
        expected_keys = set([key for key, value in expected_record])
        keys = set(self.records[0].keys())
        key_difference = expected_keys - keys
        assert not key_difference, key_difference


class TestImport:
    @classmethod
    def setup_class(self):
        self._filepath = BIS_1_FILEBASE + XL_EXTENSION
        self.importer = BisImporter(filepath=self._filepath)
        self.pkg_dicts = [pkg_dict for pkg_dict in self.importer.pkg_dict()]

    def test_0_munge(self):
        def test_munge(name, expected_munge):
            munge = self.importer.name_munge(name)
            assert munge == expected_munge, 'Got %s not %s' % (munge, expected_munge)
        test_munge('hesa-first-year-uk-domiciled-he-students-by-qualification-aim-mode-of-study-gender-and-disability-1994-1995', 'hesa-first-year-uk-domiciled-he-students-disability-1994-1995')


    def test_1_row_2_package(self):
        row_dict = OrderedDict([
            (u'Dataset Ref#', u'BIS-000002'),
            (u'Dataset Status', u'Proposed'),
            (u'Agency', u'Higher Education Statistics Agency'),
            (u'Primary Contact', u'information.provision@hesa.ac.uk'),
            (u'Secondary Contact', None),
            (u'Title', u'Higher Education Statistics: All HE students by level of study, mode of study, subject of study, domicile and gender 2007/08'),
            (u'Abstract', u'This dataset provides the 2007/08 higher education statistics for all students by level of study, mode of study, subject of study, domicile and gender'),
            (u'Date Released', 2008),
            (u'Date Updated', 2008),
            (u'Update Frequency', u'Never'),
            (u'Tags', u'hesa higher-education-statistics 2007-2008'),
            (u'Department', u'Department for Business, Innovation & Skills'),
            (u'Wiki', u'-'),
            (u'Identifier', u'higher-education-statistics-all-he-students-by-level-of-study-mode-of-study-subject-of-study-meeeeeeeeeeeeeeeeeeeeeeeeeeeega-long-name-2007-2008'),
            (u'Licence', u'Higher Education Statistcs Agency Copyright with data.gov.uk rights'),
            (u'Version', u'-'),
            (u'Geographic Coverage', u'United Kingdom (England, Scotland, Wales, Northern Ireland)'),
            (u'Geographic Granularity', u'national'),
            (u'Temporal Granularity', u'years'),
            (u'Precision', u'integer to the nearest 5'),
            (u'Taxonomy url', u'-'),
            (u'Temporal Coverage From', datetime.date(2007, 8, 1)),
            (u'Temporal Coverage To', datetime.date(2008, 7, 31)),
            (u'National Statistic', u'no'),
            (u'Categories', u'-'),
            (u'Contact information', u'Higher Education Statistics Agency (HESA)\n+44 (0) 1242 211133\ninformation.provision@hesa.ac.uk'),
            (u'Data File', 1),
            (u'Reference Material', 2),
            (u'Information', 0),
            (u'Full Description', 0),
            (u'Unknown', 0),
            (u'Total', 3)
            ])
        expected_pkg_dict = OrderedDict([
            ('name', u'hesa-all-he-students-by-level-of-study-mode-of-study-subject-of-study-meeeeeeeeeeeeee-2007-2008'),
            ('title', u'Higher Education Statistics: All HE students by level of study, mode of study, subject of study, domicile and gender 2007/08'),
            ('version', u'-'),
            ('url', None),
            ('author', u'Higher Education Statistics Agency (HESA)'),
            ('author_email', u'information.provision@hesa.ac.uk'),
            ('maintainer', u''),
            ('maintainer_email', u''),
            ('notes', u'This dataset provides the 2007/08 higher education statistics for all students by level of study, mode of study, subject of study, domicile and gender'),
            ('license_id', u'hesa-withrights'),
            ('tags', [u'2007-2008', u'education', u'hesa', \
                      u'higher-education', u'higher-education-statistics']),
            ('groups', ['ukgov']),
            ('resources', [OrderedDict([
                ('url', 'http://www.hesa.ac.uk/dox/dataTables/studentsAndQualifiers/download/subject0708.xls?v=1.0'),
                ('format', 'XLS'),
                ('description', 'Data File - XLS Format'),
                ]),
                           OrderedDict([
                               ('url', 'http://www.hesa.ac.uk/index.php/component/option,com_datatables/task,show_file/defs,1/Itemid,121/catdex,3/disp,/dld,subject0708.xls/yrStr,2007+to+2008/dfile,studefs0708.htm/area,subject/mx,0/'),
                               ('format', 'HTML'),
                               ('description', 'Reference Material - Data File Definition'),
                               ]),
                           OrderedDict([
                               ('url', 'http://www.hesa.ac.uk/index.php/component/option,com_datatables/task,show_file/defs,2/Itemid,121/catdex,3/disp,/dld,subject0708.xls/yrStr,2007+to+2008/dfile,notes0708.htm/area,subject/mx,0/'),
                               ('format', 'HTML'),
                               ('description', 'Reference Material - Notes Regarding Data File Content'),
                               ]),
                           ]),
            ('extras', OrderedDict([
                ('external_reference', 'BIS-000002'),
                ('date_released', '2008'),
                ('date_updated', '2008'),
                ('temporal_granularity', 'years'),
                ('temporal_coverage_to', '2008-07-31'),
                ('temporal_coverage_from', '2007-08-01'),
                ('geographic_coverage', '111100: United Kingdom (England, Scotland, Wales, Northern Ireland)'),
                ('geographical_granularity', 'national'),
                ('agency', u'Higher Education Statistics Agency'),
                ('precision', 'integer to the nearest 5'),
                ('taxonomy_url', '-'),
                ('import_source', 'BIS-%s' % os.path.basename(self._filepath)),
                ('department', u'Department for Business, Innovation and Skills'),
                ('update_frequency', 'Never'),
                ('national_statistic', 'no'),
                ('categories', '-'),
                ])
             ),
            ])
        pkg_dict = self.importer.row_2_package(row_dict)

        log = self.importer.get_log()
        assert not log, log

        PackageDictUtil.check_dict(pkg_dict, expected_pkg_dict)
        expected_keys = set([key for key, value in expected_pkg_dict.items()])
        keys = set(pkg_dict.keys())
        key_difference = expected_keys - keys
        assert not key_difference, key_difference

    @classmethod
    def check_dict(cls, dict_to_check, expected_dict):
        for key, value in expected_dict.items():
            if key == 'extras':
                cls.check_dict(dict_to_check['extras'], value)
            else:
                if value:
                    assert dict_to_check[key] == value, 'Key \'%s\' should be %r not: %r' % (key, value, dict_to_check[key])
                else:
                    assert not dict_to_check.get(key), 'Key \'%s\' should have no value, not: %s' % (key, dict_to_check[key])



from sqlalchemy.util import OrderedDict

from ckanext.dgu.scripts.change_licenses import ChangeLicenses
from ckan import model
from ckan.tests import *
from ckan.tests.wsgi_ckanclient import WsgiCkanClient
from ckan.lib.create_test_data import CreateTestData


class TestChangeLicenses(TestController):
    @classmethod
    def setup_class(self):
        # create test data
        CreateTestData.create()
        username = 'annafan'
        user = model.User.by_name(unicode(username))
        assert user
        self.testclient = WsgiCkanClient(self.app, api_key=user.apikey, base_location='/api/2')

    @classmethod
    def teardown_class(self):
        CreateTestData.delete()

    def get_licenses(self):
        q = model.Session.query(model.Package)
        return dict([(pkg.name, pkg.license_id) for pkg in q.all()])

    def test_1_change_all_pkgs(self):
        licenses_before = self.get_licenses()
        self.license_id = 'test-license'
        self.change_licenses = ChangeLicenses(self.testclient)
        self.change_licenses.change_all_packages(self.license_id)
        licenses_after = self.get_licenses()

        change_in_packages = set(licenses_before.keys()) - set(licenses_after.keys())
        assert not change_in_packages, change_in_packages
        for pkg_name in licenses_after.keys():
            assert licenses_after[pkg_name] == self.license_id, licenses_after[pkg_name]

    def test_2_test_change_oct_2010(self):
        # TODO
        pass
    



import os
import datetime

from ckanext.dgu.ons import downloader

class OnsDataTester(downloader.OnsData):
    '''A test version of OnsData, that uses a test harness instead of the
    real internet, to test downloading ONS data.'''

    def __init__(self, local_cache_dir=downloader.ONS_CACHE_PATH, log=False, files_downloaded=None):
        self.reset(files_downloaded)
        super(OnsDataTester, self).__init__(local_cache_dir=downloader.ONS_CACHE_PATH, log=self.test_log_func)

    def reset(self, files_downloaded=None):
        # test records
        self.logs = []
        self.files_downloaded = files_downloaded or {} # filepath: url

    def download(self, url, url_name, force_download=False):
        local_filepath = os.path.join(self._local_cache_dir, 'ons_data_%s' % url_name)
        if force_download and local_filepath in self.files_downloaded:
            del self.files_downloaded[local_filepath]
        if not local_filepath in self.files_downloaded:
            self.files_downloaded[local_filepath] = url
        else:
            self.log(logging.info, 'ONS Data already downloaded: %s' % url_name)
        return local_filepath
    
    def test_log_func(self, log_func, msg):
        self.logs.append((log_func, msg))

    def _get_today(self):
        return datetime.date(2005, 6, 21)



import sys

from ckanclient import CkanClient
from ckanext.api_command import ApiCommand
from ons_analysis import OnsAnalysis

class OnsAnalysisCommand(ApiCommand):
    def add_options(self):
        pass
    
    def command(self):
        super(OnsAnalysisCommand, self).command()

        # now do command
        client = CkanClient(base_location=self.options.api_url,
                            api_key=self.options.api_key,
                            http_user=self.options.username,
                            http_pass=self.options.password)
        change_licenses = OnsAnalysis(client)
        change_licenses.run()

def command():
    OnsAnalysisCommand().command()




from ckanext.dgu.schema import *
from nose.tools import assert_equal

class TestGeoCoverageType:
    @classmethod
    def setup_class(cls):
        cls.expected_data = [
            # str, form, db
            ('England', ['england'], u'100000: England'),
            ('England and Wales', ['england', 'wales'], u'101000: England, Wales'),
            ('Scotland', ['scotland'], u'010000: Scotland'),
            ('Northern Ireland', ['northern_ireland'], u'000100: Northern Ireland'),
            ('GB', ['england', 'scotland', 'wales'], u'111000: Great Britain (England, Scotland, Wales)'),
            ('UK', ['england', 'scotland', 'wales', 'northern_ireland'], u'111100: United Kingdom (England, Scotland, Wales, Northern Ireland)'),
            ('Sub-National', None, u'000000: '),
            ]
            
    def test_str_to_db(self):
        for str_, form, db in self.expected_data:
            if str_ is None or db is None:
                continue
            result_db = GeoCoverageType.get_instance().str_to_db(str_)
            assert_equal(result_db, db)

    def test_form_to_db(self):
        for str_, form, db in self.expected_data:
            if form is None or db is None:
                continue
            result_db = GeoCoverageType.get_instance().form_to_db(form)
            assert_equal(result_db, db)

    def test_db_to_form(self):
        for str_, form, db in self.expected_data:
            if form is None or db is None:
                continue
            result_form = GeoCoverageType.get_instance().db_to_form(db)
            assert_equal(result_form, form)

class TestAbbreviations:
    def test_basic(self):
        res = expand_abbreviations('MFA')
        assert_equal(res, 'Marine and Fisheries Agency')

class TestTags:
    def test_parse(self):
        expected_data = [
            ('pollution fish', ['pollution', 'fish']),
            ('dosh$money', ['doshmoney']),
            ('ordnance survey', ['ordnance-survey']),
            ]
        for str_, tags in expected_data:
            result_tags = tags_parse(str_)
            assert_equal(result_tags, tags)

    def test_munge(self):
        expected_data = [
            ('pollution', 'pollution'),
            ('fish pollution', 'fish-pollution'),
            ('dosh$money', 'doshmoney'),
            ('under_score', 'under-score'),
            ]
        for str_, tag in expected_data:
            result_tag = tag_munge(str_)
            assert_equal(result_tag, tag)

    def test_suggester(self):
        expected_data = [
            ({'name':'road-traffic-accident'}, ['road', 'traffic', 'accident']),
            ({'name':'road',
              'title':'Traffic accidents'}, ['road', 'traffic', 'accident']),
            ({'name':'road',
              'agency':'Traffic accidents'}, ['road', 'traffic', 'accident']),
            ({'name':'road',
              'extras':{'agency':'Traffic accidents'}}, ['road', 'traffic', 'accident']),
            ]
        for pkg_dict, tags in expected_data:
            result_tags = suggest_tags(pkg_dict)
            assert_equal(result_tags, set(tags))

class TestName:
    def test_parse(self):
        expected_data = [
            ('Annual Report', 'annual_report'),
            ('Annual Report: 2006', 'annual_report-2006'),
            ]
        for str_, name in expected_data:
            result_name = name_munge(str_)
            assert_equal(result_name, name)
    



class ScriptError(Exception):
    pass

def remove_readonly_fields(pkg):
    '''Takes a package dictionary and gets rid of any read-only fields
    so that you can write the package to the API.'''
    for read_only_field in ('id', 'relationships', 'ratings_average',
                            'ratings_count', 'ckan_url',
                            'metadata_modified',
                            'metadata_created'):
        if pkg.has_key(read_only_field):
            del pkg[read_only_field]



import datetime

from ckanext.api_command import ApiCommand
from ckanext.dgu.ons.downloader import OnsData
from ckanext.dgu.ons.importer import OnsImporter
from ckanext.dgu.ons.loader import OnsLoader
from ckanclient import CkanClient

class OnsLoaderCmd(ApiCommand):
    def add_options(self):
        self.parser.add_option("-d", "--days",
                               dest="days",
                               help="Days to fetch data (e.g. 7) (period is up to today, unless start-date or end-date specified)")
        self.parser.add_option("-s", "--start-date",
                               dest="start_date",
                               help="Date of start of period to fetch data (e.g. 2008-6-25)")
        self.parser.add_option("-e", "--end-date",
                               dest="end_date",
                               help="Date of end of period to fetch data (e.g. 2008-6-25)")
        
    def parse_date(self, date_str):
        return datetime.date(*[int(date_chunk) for date_chunk in date_str.split('-')])
    
    def command(self):
        super(OnsLoaderCmd, self).command()

        if self.options.days:
            self.options.days = int(self.options.days)
        if self.options.start_date:
            self.options.start_date = self.parse_date(self.options.start_date)
        if self.options.end_date:
            self.options.end_date = self.parse_date(self.options.end_date)
        data_filepath = OnsData.download_flexible(
            days=self.options.days,
            start_date=self.options.start_date,
            end_date=self.options.end_date)
        importer = OnsImporter(filepath=data_filepath)
        loader = OnsLoader(self.client)

        loader.load_packages(importer.pkg_dict())

def load():
    OnsLoaderCmd().command()



import re
import time
import datetime

__all__ = ['government_depts', 'geographic_granularity_options', 'temporal_granularity_options', 'category_options', 'region_options', 'region_groupings', 'update_frequency_options', 'tag_pool', 'tag_words_to_join', 'suggest_tags', 'tags_parse', 'GeoCoverageType', 'expand_abbreviations', 'name_munge', 'tag_munge']

government_depts_raw = """
Attorney General's Office
Cabinet Office
Central Office of Information
Charity Commission for England and Wales
Commissioners for the Reduction of the National Debt
Crown Estate
Crown Prosecution Service
Department for Business, Innovation and Skills
Department for Children, Schools and Families
Department for Communities and Local Government
Department for Culture, Media and Sport
Department for Education
Department for Environment, Food and Rural Affairs
Department for International Development
Department for Transport
Department for Work and Pensions
Department of Energy and Climate Change
Department of Health
Export Credits Guarantee Department
Food Standards Agency
Foreign and Commonwealth Office
Forestry Commission
Government Actuary's Department
Government Equalities Office
Her Majesty's Revenue and Customs
Her Majesty's Treasury
Home Office
Ministry of Defence
Ministry of Justice
National School of Government
Northern Ireland Office
Office for Standards in Education, Children's Services and Skills
Office of Fair Trading
Office of Gas and Electricity Markets
Office of Rail Regulation
Office of the Advocate General for Scotland
Office of the Leader of the House of Commons
Office of the Leader of the House of Lords
Office of the Parliamentary Counsel
Postal Services Commission
Public Works Loan Board
Revenue and Customs Prosecutions Office
Scotland Office
Serious Fraud Office
Treasury Solicitor's Department
UK Statistics Authority
UK Trade & Investment
Wales Office
Water Services Regulation Authority

Scottish Government
Welsh Assembly Government
Northern Ireland Executive
"""
government_depts = []
for line in government_depts_raw.split('\n'):
    if line:
        government_depts.append(unicode(line.strip()))

department_agency_abbreviations = {'DCSF':'Department for Children, Schools and Families', 'VLA':'Vetinary Laboratories Agency', 'MFA':'Marine and Fisheries Agency', 'CEFAS':'Centre of Environment, Fisheries and Aquaculture Science', 'FERA':'Food and Environment Research Agency', 'DEFRA':'Department for Environment, Food and Rural Affairs', 'Department for the Environment, Food and Rural Affairs':'Department for Environment, Food and Rural Affairs', 'CRB':'Criminal Records Bureau', 'UKBA':'UK Border Agency', 'IPS':'Identity and Passport Service', 'NPIA':'National Policing Improvement Agency', 'CIB':'Company Investigation Branch', 'IPO':'Intellectual Property Office', 'SFO':'Serious Fraud Office', 'HM Revenue and Customs':"Her Majesty's Revenue and Customs", 'HM Treasury':"Her Majesty's Treasury", 'DfT':'Department for Transport', 'socitm':'Society of Information Technology Management', 'Communities and Local government':'Department for Communities and Local Government', 'Department of Communities and Local Government':'Department for Communities and Local Government', 'Department for Education (DfE)':'Department for Education', 'HM Customs and Revenue':"Her Majesty's Revenue and Customs"}

update_frequency_options = ['never', 'discontinued', 'annual', 'quarterly', 'monthly']

geographic_granularity_options = ['national', 'regional', 'local authority', 'ward', 'point']

temporal_granularity_options = ['year', 'quarter', 'month', 'week', 'day', 'hour', 'point']

category_options = ['Agriculture and Environment', 'Business and Energy', 'Children, Education and Skills', 'Crime and Justice', 'Economy', 'Government', 'Health and Social Care', 'Labour Market', 'People and Places', 'Population', 'Travel and Transport', 'Equality and Diversity', 'Migration']

yes_no_not_yet_options = ['yes', 'no', 'not yet reviewed']

region_options = ('England', 'Scotland', 'Wales', 'Northern Ireland', 'Overseas', 'Global')

region_groupings = {'United Kingdom':['England', 'Scotland', 'Wales', 'Northern Ireland'], 'Great Britain':['England', 'Scotland', 'Wales']}

region_abbreviations = {'UK':'United Kingdom', 'N. Ireland':'Northern Ireland', 'GB':'Great Britain'}

tag_pool = ['accident', 'road', 'traffic', 'health', 'illness', 'disease', 'population', 'school', 'accommodation', 'children', 'married', 'emissions', 'benefit', 'alcohol', 'deaths', 'mortality', 'disability', 'unemployment', 'employment', 'armed forces', 'asylum', 'cancer', 'births', 'burglary', 'child', 'tax credit', 'criminal damage', 'drug', 'earnings', 'education', 'economic', 'fire', 'fraud', 'forgery', 'fuel', 'green', 'greenhouse gas', 'homeless', 'hospital', 'waiting list', 'housing', 'care', 'income', 'census', 'mental health', 'disablement allowance', 'jobseekers allowance', 'national curriculum', 'older people', 'living environment', 'higher education', 'living environment', 'school absence', 'local authority', 'carbon dioxide', 'energy', 'teachers', 'fostering', 'tide', 'gas', 'electricity', 'transport', 'veterinary', 'fishing', 'export', 'fisheries', 'pest', 'recycling', 'waste', 'crime', 'anti-social behaviour', 'police', 'refugee', 'identity card', 'immigration', 'planning', 'communities', 'lettings', 'finance', 'ethnicity', 'trading standards', 'trade', 'business', 'child protection', 'jobs']

tag_search_fields = ['name', 'title', 'notes', 'categories', 'agency']

tag_words_to_join = ['ordnance survey', 'environmental protection', 'water conservation', 'water resources', 'water quality', 'climate and weather', 'nature conservation', 'waste management', 'waste policies and regulation', 'air quality', 'tariff codes', 'life stages']

def name_munge(name):
    # convert spaces to underscores
    name = re.sub(' ', '_', name).lower()        
    # convert symbols to dashes
    name = re.sub('[:]', '_-', name)
    name = re.sub('[/]', '-', name)
    # take out not-allowed characters
    name = re.sub('[^a-zA-Z0-9-_]', '', name)
    # remove double underscores
    name = re.sub('__', '_', name)
    # fix up things like "_-_"
    name = re.sub("_-_", "-", name)
    return name[:100]

def tag_munge(name):
    '''munges a name to be suitable for a tag'''
    return name_munge(name).replace('_', '-').replace('--', '-').replace("_-_", "-")

class TagSuggester(object):
    @classmethod
    def suggest_tags(cls, pkg_dict):
        tags = set()
        for field_name in tag_search_fields:
            if pkg_dict.has_key(field_name):
                text = pkg_dict[field_name]
            else:
                if pkg_dict.has_key('extras'):
                    text = pkg_dict['extras'].get(field_name)
            if text and isinstance(text, (str, unicode)):
                text_lower = text.lower()
                for keyword in tag_pool:
                    if keyword in text_lower:
                        tags.add(tag_munge(keyword))
        return tags

suggest_tags = TagSuggester.suggest_tags

def tags_parse(tags_str):
    '''Takes a string containing tags and returns a list of tag names.
    Takes into account tag_words_to_join. Ensures tags are munged.'''
    tags_str = tags_str.lower()
    for phrase in tag_words_to_join:
        if phrase in tags_str:
            tags_str = tags_str.replace(phrase, tag_munge(phrase))
    tag_list = re.split(',\s*|\s+', tags_str.strip())
    return [tag_munge(tag_name) for tag_name in tag_list]

    
class GeoCoverageType(object):
    @staticmethod
    def get_instance():
        if not hasattr(GeoCoverageType, 'instance'):
            GeoCoverageType.instance = GeoCoverageType.Singleton()
        return GeoCoverageType.instance

    class Singleton(object):
        def __init__(self):
            regions_str = region_options
            self.groupings = region_groupings
            self.regions = [(region_str, GeoCoverageType.munge(region_str)) for region_str in regions_str]
            self.regions_munged = [GeoCoverageType.munge(region_str) for region_str in regions_str]

        def munged_regions_to_printable_region_names(self, munged_regions):
            incl_regions = []
            for region_str, region_munged in self.regions:
                if region_munged in munged_regions:
                    incl_regions.append(region_str)
            for grouping_str, regions_str in self.groupings.items():
                all_regions_in = True
                for region_str in regions_str:
                    if region_str not in incl_regions:
                        all_regions_in = False
                        break
                if all_regions_in:
                    for region_str in regions_str:
                        incl_regions.remove(region_str)
                    incl_regions.append('%s (%s)' % (grouping_str, ', '.join(regions_str)))
            return ', '.join(incl_regions)

        def str_to_db(self, regions_str):
            for abbrev, region in region_abbreviations.items():
                regions_str = regions_str.replace(abbrev, region)
            for grouping, regions in region_groupings.items():
                regions_str = regions_str.replace(grouping, ' '.join(regions))
            regions_munged = []
            for region, region_munged in self.regions:
                if region in regions_str:
                    regions_munged.append(region_munged)
            return self.form_to_db(regions_munged)

        def form_to_db(self, form_regions):
            assert isinstance(form_regions, list)
            coded_regions = u''
            for region_str, region_munged in self.regions:
                coded_regions += '1' if region_munged in form_regions else '0'
            regions_str = self.munged_regions_to_printable_region_names(form_regions)
            return '%s: %s' % (coded_regions, regions_str)

        def db_to_form(self, form_regions):
            '''
            @param form_regions e.g. 110000: England, Scotland
            @return e.g. ["england", "scotland"]
            '''
            regions = []
            if len(form_regions)>len(self.regions):
                for i, region in enumerate(self.regions):
                    region_str, region_munged = region
                    if form_regions[i] == '1':
                        regions.append(region_munged)
            return regions

    @staticmethod
    def munge(region):
        return region.lower().replace(' ', '_')

    def __getattr__(self, name):
        return getattr(self.instance, name)

def expand_abbreviations(dept):
    for brief_form in department_agency_abbreviations.keys():
        if brief_form in dept:
            dept = dept.replace(brief_form,
                                department_agency_abbreviations[brief_form])
    return dept



from ckan.lib.create_test_data import CreateTestData

def teardown_module():
    assert not CreateTestData.get_all_data(), 'A test in module %r forgot to clean-up its data: %r' % (__name__, CreateTestData.get_all_data())



import os.path
import datetime
import re
import logging

from collections import defaultdict

from sqlalchemy.util import OrderedDict

from ckan.lib.importer import RowParseError, ImportException
from ckan.lib.spreadsheet_importer import CsvData, XlData, SpreadsheetDataRecords, MultipleSpreadsheetDataRecords, SpreadsheetPackageImporter
from ckan.lib import schema_gov
from ckan.lib import field_types
from ckan import model

log = logging.getLogger(__name__)

class CospreadDataRecords(SpreadsheetDataRecords):
    def __init__(self, data):
        essential_title = 'Package name'
        self.column_name_map = {
            u'tags':'Tags',
            u'Download file format':'File format',
            u'Download description':'Download Description',
            u'Maintainer - E-mail address, if needed.':u'Maintainer - E-mail address',
            u'Maintainer - Blank unless not the author.':u'Maintainer - ',
            u'CO Reference':u'CO Identifier',
            u'Author - E-mail address.':'Contact - E-mail address.',
            u'Author - Permanent contact point for members of the public; not an individual.':'Contact - Permanent contact point',
            }
        self.column_name_reverse_map = dict((v, k) for k, v in self.column_name_map.iteritems())
        self.optional_columns = [u'Temporal Coverage - To\n(if needed)',
                                 u'Temporal Coverage - From',
                                 u'Download Description',
                                 u'National Statistic',
                                 u'Maintainer - E-mail address',
                                 u'Maintainer - ',
                                 u'Categories']
        self.column_spreading_titles = ['Geographical Granularity', 'Geographic coverage', 'Temporal Granularity', 'Temporal Coverage', 'Author', 'Maintainer', 'Contact']
        self.standard_or_other_columns = ['Geographical Granularity', 'Temporal Granularity']
        self.resource_keys = ['Download URL', 'File format', 'Download Description']
        super(CospreadDataRecords, self).__init__(data, essential_title)
            
    def find_titles(self, essential_title):
        row_index = 0
        titles = []
        essential_title_lower = essential_title.lower()
        while True:
            if row_index >= self._data.get_num_rows():
                raise ImportException('Could not find title row')
            row = self._data.get_row(row_index)
            if essential_title in row or essential_title_lower in row:
                next_row = self._data.get_row(row_index + 1)
                last_title = None
                for col_index, row_val in enumerate(row):
                    if not row_val:
                        title = None
                        if last_title in self.column_spreading_titles:
                            title = '%s - %s' % (last_title, next_row[col_index])
                    else:
                        title = row_val.strip().replace('  ', ' ')
                        last_title = title
                    if title in self.column_spreading_titles:
                        title = '%s - %s' % (title, next_row[col_index])
                    titles.append(title)
                return (titles, row_index + 1)
            row_index += 1

    @property
    def records(self):
        '''Returns package records.
        * Collates packages with download_url in multiple rows in resources.
        * Collapses 'Standard' / 'Other' column pairs into single value.
        '''
        current_record = None
        for record in super(CospreadDataRecords, self).records:
            if current_record and current_record['Package name'] == record['Package name']:
                # this record is another resource for the current record.
                keys_that_should_match = set(current_record.keys()) - set(self.resource_keys + ['resources'] + self.standard_or_other_columns)
                for key in keys_that_should_match:
                    alt_key = self.column_name_reverse_map.get(key)
                    if alt_key and alt_key in record:
                        record_key = alt_key
                    else:
                        record_key = key
                    assert current_record[key] == record[record_key], 'Multiple resources for package %s, but value does not match: %r!=%r' % (record['Package name'], current_record[key], record[key])
            else:
                # this record is new, so yield the old 'current_record' before
                # making this record 'current_record'.
                if current_record:
                    yield current_record
                current_record = record.copy()
                current_record['resources'] = []
                # Collapse standard/other columns into one
                for column in self.standard_or_other_columns:
                    standard = current_record['%s - Standard' % column]
                    other = current_record['%s - Other' % column]
                    if standard == 'Other (specify)' or standard == None:
                        value = other
                    else:
                        assert not other, 'Both "Standard" and "Other" values for column %r in record %r' % (column, current_record)
                        value = standard
                    current_record[column] = value
                    del current_record['%s - Standard' % column]
                    del current_record['%s - Other' % column]
                # Rename columns to standard names
                keys_to_change = set(current_record.keys()) & set(self.column_name_map.keys())
                for from_key in keys_to_change:
                    to_key = self.column_name_map[from_key]
                    current_record[to_key] = current_record[from_key]
                    del current_record[from_key]
            # Put download_url into resources
            resource = OrderedDict()
            for key in self.resource_keys:
                key_used = None
                if key in record:                
                    key_used = key
                else:
                    alt_key = self.column_name_reverse_map.get(key)
                    if alt_key and alt_key in record:
                        key_used = alt_key
                        value = record[alt_key]
                if key_used:
                    value = record[key_used]
                    resource[key] = value
                    del record[key_used]
                else:
                    if key in self.optional_columns:
                        record[key] = None
                    else:
                        raise KeyError(key)
            current_record['resources'].append(resource)
        if current_record:
            yield current_record

class MultipleCospreadDataRecords(MultipleSpreadsheetDataRecords):
    def __init__(self, data):
        if data.get_num_sheets() > 1:
            data = data.get_data_by_sheet()
        super(MultipleCospreadDataRecords, self).__init__(data, [], record_class=CospreadDataRecords)


class CospreadImporter(SpreadsheetPackageImporter):
    def __init__(self, include_given_tags=False, **kwargs):
        self.include_given_tags = include_given_tags
        self.license_map = {
            u'UK Crown Copyright with data.gov.uk rights':u'uk-ogl',
            u'\xa9 HESA. Not core Crown Copyright.':u'uk-ogl',
            u'Local Authority copyright with data.gov.uk rights':u'uk-ogl',
            u'Local Authority Copyright with data.gov.uk rights':u'uk-ogl',
            u'UK Crown Copyright':u'uk-ogl',
            u'Crown Copyright':u'uk-ogl', }
        super(CospreadImporter, self).__init__(record_params=[], record_class=CospreadDataRecords, **kwargs)

    def log(self, msg):
        super(CospreadImporter, self).log(msg)
        log.warn(msg)

    def record_2_package(self, row_dict):
        pkg_dict = OrderedDict()
        pkg_dict['title'] = row_dict['Title']
        pkg_dict['name'] = self.name_munge(row_dict.get('Package name') or u'') or self.munge(pkg_dict['title'])
        if not (pkg_dict['name'] and pkg_dict['title']):
            raise RowParseError('Both Name and Title fields must be filled: name=%r title=%r' % (pkg_dict['name'], pkg_dict['title']))
        pkg_dict['author'] = row_dict['Contact - Permanent contact point']
        pkg_dict['author_email'] = row_dict['Contact - E-mail address.']
        pkg_dict['maintainer'] = row_dict['Maintainer - ']
        pkg_dict['maintainer_email'] = row_dict['Maintainer - E-mail address']
        notes = row_dict['Notes']
        license_id = self.license_map.get(row_dict['Licence'].strip(), '')
        if not license_id and ';' in row_dict['Licence']:
            license_parts = row_dict['Licence'].split(';')
            for i, license_part in enumerate(license_parts):
                license_id = license_map.get(license_part.strip(), '')
                if license_id:
                    notes += '\n\nLicence detail: %s' % row_dict['Licence']
                    break
        if not license_id:
            license_id = 'uk-ogl'
            self.log('WARNING: license not recognised: "%s". Defaulting to: %s.' % (row_dict['Licence'], license_id))
        pkg_dict['license_id'] = license_id
        pkg_dict['url'] = self.tidy_url(row_dict['URL'])
        pkg_dict['notes'] = notes
        pkg_dict['version'] = u''
        pkg_dict['groups'] = [u'ukgov']

        pkg_dict['extras'] = OrderedDict()
        extras_dict = pkg_dict['extras']
        geo_cover = []
        geo_coverage_type = schema_gov.GeoCoverageType.get_instance()
        spreadsheet_regions = ('England', 'N. Ireland', 'Scotland', 'Wales', 'Overseas', 'Global')
        for region in spreadsheet_regions:
            munged_region = region.lower().replace('n. ', 'northern_')
            field = 'Geographic coverage - %s' % region
            if (row_dict[field] or '').lower() not in (None, '', 'no', 'False'):
                geo_cover.append(munged_region)
        extras_dict['geographic_coverage'] = geo_coverage_type.form_to_db(geo_cover)

        for column, extra_key in [
            ('Date released', 'date_released'),
            ('Date updated', 'date_updated'),
            ('Temporal Coverage - From', 'temporal_coverage_from'),
            ('Temporal Coverage - To\n(if needed)', 'temporal_coverage_to'),
            ]:
            form_value = row_dict.get(column)
            if isinstance(form_value, datetime.date):
                val = field_types.DateType.date_to_db(form_value)
            else:
                if isinstance(form_value, int):
                    form_value = str(form_value)
                # Hack for CLG data to allow '2008/09' to mean '2008', or
                # '2009' if it is a 'To' field.
                match = re.match('(\d{4})/(\d{2})', form_value or '')
                if match:
                    years = [int(year_str) for year_str in match.groups()]
                    if extra_key.endswith('_to'):
                        form_value = str(field_types.DateType.add_centurys_to_two_digit_year(year=years[1], near_year=years[0]))
                    else:
                        form_value = str(years[0])
                try:
                    val = field_types.DateType.form_to_db(form_value)
                except field_types.DateConvertError, e:
                    self.log("WARNING: Value for column '%s' of %r is not understood as a date format." % (column, form_value))
                    val = form_value
            extras_dict[extra_key] = val
            
        field_map = [
            ['CO Identifier'],
            ['Update frequency'],
            ['Temporal Granularity', schema_gov.temporal_granularity_options],
            ['Geographical Granularity', schema_gov.geographic_granularity_options],
            ['Categories', schema_gov.category_options],
            ['Taxonomy URL'],
            ['Agency responsible'],
            ['Precision'],
            ['Department', schema_gov.government_depts],
            ]
        optional_fields = ['Categories']
        for field_mapping in field_map:
            column = field_mapping[0]
            extras_key = column.lower().replace(' ', '_')
            if column == 'Agency responsible':
                extras_key = 'agency'
            elif column in ('CO Identifier', 'CO Reference'):
                if row_dict.has_key('CO Reference'):
                    column = 'CO Reference'
                extras_key = 'external_reference'
            if row_dict.has_key(column):
                val = row_dict[column]
            else:
                assert column in optional_fields, column
                val = None
            if len(field_mapping) > 1:
                # snap to suggestions
                suggestions = field_mapping[1]
                if val and val not in suggestions:
                    suggestions_lower = [sugg.lower() for sugg in suggestions]
                    if val.lower() in suggestions_lower:
                        val = suggestions[suggestions_lower.index(val.lower())]
                    elif schema_gov.expand_abbreviations(val) in suggestions:
                        val = schema_gov.expand_abbreviations(val)
                    elif val.lower() + 's' in suggestions:
                        val = val.lower() + 's'
                    elif val.replace('&', 'and').strip() in suggestions:
                        val = val.replace('&', 'and').strip()
                if val and val not in suggestions:
                    self.log("WARNING: Value for column '%s' of '%s' is not in suggestions '%s'" % (column, val, suggestions))
            extras_dict[extras_key] = val
        
        extras_dict['national_statistic'] = u'' # Ignored: row_dict['national statistic'].lower()
        extras_dict['import_source'] = 'COSPREAD-%s' % os.path.basename(self._filepath)

        resources = []
        for row_resource in row_dict['resources']:
            res_dict = OrderedDict([
                ('url', self.tidy_url(row_resource['Download URL'])),
                ('format', row_resource.get('File format', u'')),
                ('description', row_resource.get('Download Description', u'')),
                ])
            if '\n' in res_dict['url']:
                # multiple urls
                for url in res_dict['url'].split():
                    res_dict_tmp = OrderedDict(res_dict.items()) # i.e. deepcopy
                    res_dict_tmp['url'] = url
                    resources.append(res_dict_tmp)
            else:
                resources.append(res_dict)
        pkg_dict['resources'] = resources

        tags = schema_gov.TagSuggester.suggest_tags(pkg_dict)
        if self.include_given_tags:
            given_tags = schema_gov.tags_parse(row_dict['Tags'])
            tags = tags | set(given_tags)
        pkg_dict['tags'] = sorted(list(tags))

        return pkg_dict
        
    @classmethod
    def munge(self, name):
        # convert spaces and separating symbols to underscores
        name = re.sub('[\s:/]', '-', name).lower()        
        # take out not-allowed characters
        name = re.sub('[^a-z0-9-_]', '', name)
        # remove double minuses
        name = re.sub('--', '-', name)
        # if longer than max_length, keep last word if a year
        max_length = model.PACKAGE_NAME_MAX_LENGTH
        if len(name) > max_length:
            year_match = re.match('.*?[_-]((?:\d{2,4}[-/])?\d{2,4})$', name)
            if year_match:
                year = year_match.groups()[0]
                name = '%s-%s' % (name[:(max_length-len(year)-1)], year)
            else:
                name = name[:max_length]
        return name

    def name_munge(self, input_name):
        '''Munges the name field in case it is not to spec.'''
        input_name = input_name.replace(' ', '').replace('.', '_').replace('&', 'and')
        return super(CospreadImporter, self).name_munge(input_name)

    def tidy_url(self, url):
        if url and not url.startswith('http') and not url.startswith('webcal:'):
            if url.startswith('www.'):
                url = url.replace('www.', 'http://www.')
            else:
                self.log("WARNING: URL doesn't start with http: %s" % url)
        return url



import copy

from ckan import model
from common import ScriptError, remove_readonly_fields

log = __import__("logging").getLogger(__name__)

class PackageMatcher(object):
    def match_ref(self, pkg_name):
        '''Override this and return True or False depending on whether
        the supplied package matches or not. If the matcher requires
        more package information, then return None and the match method
        is run.'''
        pass

    def match(self, pkg):
        '''Override this and return True or False depending on whether
        the supplied package matches or not.'''
        assert NotImplementedError

class BasicPackageMatcher(PackageMatcher):
    def __init__(self, match_field, match_field_value):
        '''Package matching criteria: match_field==match_field_value'''
        self.match_field = match_field
        self.match_field_value = match_field_value
        
    def match(self, pkg):
        value = pkg.get(self.match_field) or \
                pkg['extras'].get(self.match_field)
        if value == self.match_field_value:
            return True
        else:
            return False

class AnyPackageMatcher(PackageMatcher):
    def match(self, pkg):
        return True

class ListedPackageMatcher(PackageMatcher):
    '''Package matcher based on a supplied list of package names.'''
    def __init__(self, pkg_name_list):
        assert iter(pkg_name_list)
        assert not isinstance(pkg_name_list, basestring)
        self.pkg_name_list = set(pkg_name_list)
        
    def match_name(self, pkg_name):
        return pkg_name in self.pkg_name_list

    def match(self, pkg):
        return pkg['name'] in self.pkg_name_list

class PackageChanger(object):
    def change(self, pkg):
        '''Override this to return a changed package dictionary.
        @param pkg: package dictionary
        '''
        assert NotImplementedError

    def resolve_field_value(self, input_field_value, pkg):
        '''Resolves the specified field_value to one
        specific to this package.
        Examples:
          "pollution" -> "pollution"
          "%(name)s" -> "uk-pollution-2008"
        '''
        return input_field_value % pkg

    def flatten_pkg(self, pkg_dict):
        flat_pkg = copy.deepcopy(pkg_dict)
        for name, value in pkg_dict.items()[:]:
            if isinstance(value, (list, tuple)):
                if value and isinstance(value[0], dict) and name == 'resources':
                    for i, res in enumerate(value):
                        prefix = 'resource-%i' % i
                        flat_pkg[prefix + '-url'] = res['url']
                        flat_pkg[prefix + '-format'] = res['format']
                        flat_pkg[prefix + '-description'] = res['description']
                else:
                    flat_pkg[name] = ' '.join(value)
            elif isinstance(value, dict):
                for name_, value_ in value.items():
                    flat_pkg[name_] = value_
            else:
                flat_pkg[name] = value
        return flat_pkg
        

class BasicPackageChanger(PackageChanger):
    def __init__(self, change_field, change_field_value):
        '''Changes: pkg.change_field = change_field_value'''
        self.change_field = change_field
        self.change_field_value = change_field_value
        
    def change(self, pkg):
        flat_pkg = self.flatten_pkg(pkg)
        if pkg.has_key(self.change_field):
            pkg_field_root = pkg
        else:
            pkg_field_root = pkg['extras']
        value = self.resolve_field_value(self.change_field_value, flat_pkg)

        log.info('%s.%s  Value %r -> %r' % \
                 (pkg['name'], self.change_field,
                  flat_pkg.get(self.change_field),
                  value))

        pkg_field_root[self.change_field] = value
        return pkg

class CreateResource(PackageChanger):
    def __init__(self, **resource_values):
        '''Adds new resource with the given values.
        @param resources_values: resource dictionary. e.g.:
                {'url'=xyz, 'description'=xyz}
        '''
        for key in resource_values.keys():
            assert key in model.PackageResource.get_columns()
        self.resource_values = resource_values
        
    def change(self, pkg):
        flat_pkg = self.flatten_pkg(pkg)
        resource = {}
        for key, value in self.resource_values.items():
            resource[key] = self.resolve_field_value(value, flat_pkg)
        resource_index = len(pkg['resources'])
        
        log.info('%s.resources[%i] -> %r' % \
                 (pkg['name'], resource_index, resource))

        pkg['resources'].append(resource)
        return pkg

class NoopPackageChanger(PackageChanger):
    def change(self, pkg):
        log.info('%s  No change' % \
                 (pkg['name']))
        return pkg

class ChangeInstruction(object):
    def __init__(self, matchers=None, changers=None):
        '''Finds packages matching criteria and changes them as specified.
        Matchers are derived from PackageMatcher and any of the matchers
        can match to apply all the changes, which derive from PackageChanger.
        '''
        if isinstance(matchers, PackageMatcher):
            self.matchers = [matchers]
        elif isinstance(matchers, list) or matchers == None:
            self.matchers = matchers
        if isinstance(changers, PackageChanger):
            self.changers = [changers]
        elif isinstance(changers, list) or changers == None:
            self.changers = changers


class MassChanger(object):
    def __init__(self, ckanclient, instructions, dry_run=False, force=False):
        '''
        Changes package properties en masse
        @param ckanclient: instance of ckanclient to make the changes
        @param instructions: (ordered) list of ChangeInstruction objects
        @param dry_run: show matching and potential changes, but do not
                        write the changes back to the server.
        @param force: prevents aborting when there is an error with one package
        '''
        self.ckanclient = ckanclient
        version = self.ckanclient.api_version_get()
        assert int(version) >= 2, 'API Version is %s. Script requires at least Version 2.' % version
        self.instructions = instructions
        self.dry_run = dry_run
        self.force = force
        self._pkg_cache = {}

    def run(self):
        pkg_refs = self.ckanclient.package_register_get()
        for pkg_ref in pkg_refs:
            try:
                pkg = self._get_pkg(pkg_ref)
                instruction = self._match_instructions(pkg_ref)
                if instruction:
                    self._change_package(self._get_pkg(pkg_ref), instruction)
            except ScriptError, e:
                err = 'Problem with package %s: %r' % (pkg_ref, e.args)
                log.error(err)
                if not self.force:
                    log.error('Aborting (avoid this with --force)')
                    raise ScriptError(err)

    def _get_pkg(self, pkg_ref):
        if not self._pkg_cache.has_key(pkg_ref):
            pkg = self.ckanclient.package_entity_get(pkg_ref)
            if self.ckanclient.last_status != 200:
                raise ScriptError('Could not get package ID %s: %r' % \
                      (pkg_ref, self.ckanclient.last_status))
            remove_readonly_fields(pkg)
            self._pkg_cache[pkg_ref] = pkg
        return self._pkg_cache[pkg_ref]
                           
    def _match_instructions(self, pkg_ref):
        for instruction in self.instructions:
            for matcher in instruction.matchers:
                assert isinstance(matcher, PackageMatcher), matcher
                name_match = None
#                name_match = matcher.match_name(pkg_ref)
                if name_match == None:
                    pkg = self._get_pkg(pkg_ref)
                    if matcher.match(pkg):
                        return instruction

    def _change_package(self, pkg, instruction):
        pkg_before = copy.deepcopy(pkg)
        for changer in instruction.changers:
            pkg = changer.change(pkg)

        if pkg == pkg_before:
            log.info('...not changed')
            return
        
        if not self.dry_run:
            self.ckanclient.package_entity_put(pkg)
            if self.ckanclient.last_status == 200:
                log.info('...saved %s:' % pkg['name'])
                log.debug('Package saved: %r' % self.ckanclient.last_message)
            else:
                raise ScriptError('Post package %s error: %s' % (pkg['name'], self.ckanclient.last_message))


class MassChangerNamedPackages(MassChanger):
    def run(self):
        for pkg_ref in self.pkg_name_list:
            try:
                instruction = self._match_instructions(pkg_ref)
                if instruction:
                    self._change_package(self._get_pkg(pkg_ref), instruction)
            except ScriptError, e:
                err = 'Problem with package %s: %r' % (pkg_ref, e.args)
                log.error(err)
                if not self.force:
                    log.error('Aborting (avoid this with --force)')
                    raise ScriptError(err)



import sys

#from ckanext.api_command import ApiCommand
from mass_changer_cmd import MassChangerCommand
from ofsted_fix import OfstedFix

class OfstedFixCmd(MassChangerCommand):
    def command(self):
        super(OfstedFixCmd, self).command()

        # now do command
        cmd = OfstedFix(self.client, dry_run=self.options.dry_run)
        cmd.run()

def command():
    OfstedFixCmd().command()




import sys

from ckanext.loader import ResourceSeries
from ckanext.dgu.scripts.change_licenses import ChangeLicenses
from ckanext.dgu.scripts.mass_changer_cmd import MassChangerCommand
from ckanclient import CkanClient

class ChangeLicensesCommand(MassChangerCommand):
    def __init__(self):
        commands = ('all', 'oct10')
        super(ChangeLicensesCommand, self).__init__(commands)

    def add_options(self):
        self.parser.add_option("--license-id",
                               dest="license_id",
                               help="ID of the license to change all packages to")
                
    def command(self):
        super(ChangeLicensesCommand, self).command()
        if self.options.license_id is None:
            self.parser.error("Please specify a license ID")
        if len(self.args) != 1:
            self.parser.error("Command is required")
        
        getattr(self, self.args[0])()

    def all(self):
        client = CkanClient(base_location=self.options.api_url,
                            api_key=self.options.api_key,
                            http_user=self.options.username,
                            http_pass=self.options.password)
        change_licenses = ChangeLicenses(client, dry_run=self.options.dry_run, force=self.options.force)
        change_licenses.change_all_packages(self.options.license_id)

    def oct10(self):
        client = CkanClient(base_location=self.options.api_url,
                            api_key=self.options.api_key)
        change_licenses = ChangeLicenses(client, dry_run=self.options.dry_run, force=self.options.force)
        change_licenses.change_oct_2010(self.options.license_id)

def command():
    ChangeLicensesCommand().command()




from ckanext.loader import ResourceSeries
from ckanext.dgu.scripts.transfer_url import TransferUrl
from ckanext.dgu.scripts.mass_changer_cmd import MassChangerCommand
from ckanclient import CkanClient

class TransferUrlCommand(MassChangerCommand):
    def command(self):
        super(TransferUrlCommand, self).command()
        if self.options.license_id is None:
            self.parser.error("Please specify a license ID")
        if len(self.args) != 1:
            self.parser.error("Command is required")
            
        client = CkanClient(base_location=self.options.api_url,
                            api_key=self.options.api_key,
                            http_user=self.options.username,
                            http_pass=self.options.password)
        transfer_url = TransferUrl(client, dry_run=self.options.dry_run,
                                   force=self.options.force)
        transfer_url.transfer_url()

def command():
    TransferUrlCommand().command()




import os
import datetime

from nose.tools import assert_equal

from ons_data_tester import OnsDataTester
from ckanext.dgu.ons import downloader

class TestOnsData:
    def __init__(self):
        self.ons_cache_path = os.path.expanduser(downloader.ONS_CACHE_PATH)
        self.ons_url_base = downloader.ONS_URL_BASE[:downloader.ONS_URL_BASE.find('?')]
        
    def test_get_url(self):
        ons_data = OnsDataTester()
        res = ons_data._get_url(31, 12, 2004, 30, 6, 2005)
        assert res[0] == self.ons_url_base + '?lday=31&lmonth=12&lyear=2004&uday=30&umonth=6&uyear=2005', res[0]
        assert res[1] == '2004-12-31_-_2005-6-30', res[1]

    def test_get_url_month(self):
        ons_data = OnsDataTester()
        res = ons_data._get_url_month(12, 2004)
        assert res[0] == self.ons_url_base + '?lday=1&lmonth=12&lyear=2004&uday=31&umonth=12&uyear=2004', res[0]
        assert res[1] == '2004-12', res[1]

    def test_get_url_recent(self):
        ons_data = OnsDataTester()
        res = ons_data._get_url_recent(days=7)
        assert res[0] == self.ons_url_base + '?lday=14&lmonth=06&lyear=2005&uday=21&umonth=06&uyear=2005', res[0]
        assert res[1] == '7_days_to_2005-06-21', res[1]

    def test_get_url_recent_via_flexible(self):
        ons_data = OnsDataTester()
        res = ons_data._get_url_flexible(days=7)
        assert res[0] == self.ons_url_base + '?lday=14&lmonth=06&lyear=2005&uday=21&umonth=06&uyear=2005', res[0]
        assert res[1] == '7_days_to_2005-06-21', res[1]

    def test_get_url_end_date(self):
        ons_data = OnsDataTester()
        res = ons_data._get_url_flexible(
            days=7, end_date=datetime.date(2005, 6, 15))
        assert_equal(res[0], self.ons_url_base + '?lday=08&lmonth=06&lyear=2005&uday=15&umonth=06&uyear=2005')
        assert_equal(res[1], '7_days_to_2005-06-15')

    def test_get_url_start_date(self):
        ons_data = OnsDataTester()
        res = ons_data._get_url_flexible(
            days=7, start_date=datetime.date(2005, 6, 7))
        assert_equal(res[0], self.ons_url_base + '?lday=07&lmonth=06&lyear=2005&uday=14&umonth=06&uyear=2005')
        assert_equal(res[1], '7_days_from_2005-06-07')

    def test_get_url_period(self):
        ons_data = OnsDataTester()
        res = ons_data._get_url_flexible(
            start_date=datetime.date(2005, 6, 7),
            end_date=datetime.date(2005, 6, 9))
        assert_equal(res[0], self.ons_url_base + '?lday=07&lmonth=06&lyear=2005&uday=09&umonth=06&uyear=2005')
        assert_equal(res[1], '2005-06-07_to_2005-06-09')

    def test_get_urls_for_all_time(self):
        ons_data = OnsDataTester()
        url_tuples = ons_data._get_urls_for_all_time()
        assert len(url_tuples) == 18, len(url_tuples)
        assert url_tuples[0] == [self.ons_url_base + '?lday=1&lmonth=1&lyear=2004&uday=31&umonth=1&uyear=2004', '2004-01', False], url_tuples[0]
        assert url_tuples[-2] == [self.ons_url_base + '?lday=1&lmonth=5&lyear=2005&uday=31&umonth=5&uyear=2005', '2005-05', False], url_tuples[-2]
        assert url_tuples[-1] == [self.ons_url_base + '?lday=1&lmonth=6&lyear=2005&uday=31&umonth=6&uyear=2005', '2005-06_incomplete', True], url_tuples[-1]


    def test_download(self):
        url = 'testurl'
        url_name = 'UrlName'
        ons_data = OnsDataTester()
        res = ons_data.download(url, url_name, force_download=False)
        assert res == self.ons_cache_path + '/ons_data_UrlName', res
        assert ons_data.files_downloaded == {res: 'testurl'}, res.items()

    def _test_import_recent(self):
        res = OnsDataTester.import_recent(days=7)
        assert res == 5, res



from ckanext.api_command import ApiCommand
from ckanext.dgu.cospread.cospread import CospreadImporter
from ckanext.dgu.cospread.loader import CospreadLoader
from ckanclient import CkanClient

class CospreadCommand(ApiCommand):
    def __init__(self):
        usage = 'usage: %prog [options] {metadata.xls}'
        super(CospreadCommand, self).__init__(usage=usage)

    def add_options(self):
        self.parser.add_option("-t", "--include-given-tags",
                               dest="include_given_tags",
                               action="store_true",
                               default=False,
                               help="Uses source's tags (in addition to just common keywords found.)")

    def command(self):
        super(CospreadCommand, self).command()

        if len(self.args) != 1:
            self.parser.error('You must specify metadata file')
        else:
            data_filepath = self.args[0]
        importer = CospreadImporter(filepath=data_filepath,
                                    include_given_tags=self.options.include_given_tags)
        
        loader = CospreadLoader(self.client)
        loader.load_packages(importer.pkg_dict())

def load():
    CospreadCommand().command()



import os.path

from collections import defaultdict

from sqlalchemy.util import OrderedDict

from ckanext.loader import *
from ckan.lib.importer import RowParseError
from ckan.lib.spreadsheet_importer import XlData, SpreadsheetDataRecords, SpreadsheetPackageImporter
from ckan.lib import schema_gov
from ckan.lib import field_types
from ckan import model


class BisLoader(PackageLoader):
    def __init__(self, ckanclient):
        settings = ReplaceByExtraField('external_reference')
        super(BisLoader, self).__init__(ckanclient, settings)


class BisImporter(SpreadsheetPackageImporter):
    def import_into_package_records(self):
        package_data = XlData(self.log, filepath=self._filepath,
                                      buf=self._buf,
                                      sheet_index=0)
        sheet_names = package_data.get_sheet_names()
        assert sheet_names == ['Datasets', 'Resources']
        resource_data = XlData(self.log, filepath=self._filepath,
                                      buf=self._buf,
                                      sheet_index=sheet_names.index('Resources'))

        self._package_data_records = SpreadsheetDataRecords(package_data, 'Title')
        self.import_resources(resource_data)

    def import_resources(self, resource_data):
        resource_data_records = SpreadsheetDataRecords(resource_data, 'Resource Title')
        self._resources_by_ref = defaultdict(list)
        for resource_record in resource_data_records.records:
            ref = resource_record['Dataset Ref#']
            resource_dict = self.row_2_resource(resource_record)
            self._resources_by_ref[ref].append(resource_dict)

    def row_2_resource(self, row_dict):
        url = self.tidy_url(row_dict['Resource url'], self.log)
        resource_dict = OrderedDict([
            ('url', url),
            ('format', row_dict['Resource Format']),
            ('description', row_dict['Resource Title']),
            ])
        return resource_dict

    def row_2_package(self, row_dict):
        name = (row_dict.get('Identifier') or u'').replace('higher-education-statistics', 'hesa')
        name = self.name_munge(name)
        title = row_dict['Title']
        if not (name and title):
            raise RowParseError('Both Name and Title fields must be filled: name=%r title=%r' % (name, title))
        contacts = row_dict['Contact information'].split('\n')
        if len(contacts) != 3:
            raise RowParseError('Unknown contacts format with %i line(s) not 3:' % (len(contacts), contact_information))
        author, ignore_phone, author_email = contacts

        license_name = row_dict['Licence'].replace('Statistcs', 'Statistics')
        license_id = self.license_2_license_id(license_name, self.log)
        if not license_id:
            raise RowParseError('No license recognised for: %r' % license_name)

        ref = row_dict['Dataset Ref#']
        if not ref.startswith('BIS-'):
            raise RowParseError('Reference must start with \'BIS-\': %s' % ref)
        resources = self._resources_by_ref[ref]

        geo_coverage = schema_gov.GeoCoverageType.get_instance().str_to_db(row_dict['Geographic Coverage'])

        munged_dates = {}
        for column in ['Date Released', 'Date Updated',
                       'Temporal Coverage To', 'Temporal Coverage From']:
            val = '%s' % row_dict[column]
            munged_dates[column] = val

        taxonomy_url = row_dict['Taxonomy url']
        if taxonomy_url and taxonomy_url != '-':
            taxonomy_url = self.tidy_url(taxonomy_url, self.log)
            
        national_statistic = u'no'
        if row_dict['National Statistic'] != national_statistic:
            self.log('Warning: Ignoring national statistic for non-ONS data: %s' % row_dict['National Statistic'])

        pkg_dict = OrderedDict([
            ('name', name),
            ('title', title),
            ('version', row_dict['Version'][:model.PACKAGE_VERSION_MAX_LENGTH]),
            ('url', None),
            ('author', author),
            ('author_email', author_email),
            ('maintainer', None),
            ('maintainer_email', None),
            ('notes', row_dict['Abstract']),
            ('license_id', license_id),
            ('tags', []), # post-filled
            ('groups', ['ukgov']),
            ('resources', resources),
            ('extras', OrderedDict([
                ('external_reference', ref),
                ('date_released', munged_dates['Date Released']),
                ('date_updated', munged_dates['Date Updated']),
                ('temporal_granularity', row_dict['Temporal Granularity']),
                ('temporal_coverage_from', munged_dates['Temporal Coverage From']),
                ('temporal_coverage_to', munged_dates['Temporal Coverage To']),
                ('geographic_coverage', geo_coverage),
                ('geographical_granularity', row_dict['Geographic Granularity']),
                ('agency', row_dict['Agency']),
                ('precision', row_dict['Precision']),
                ('taxonomy_url', taxonomy_url),
                ('import_source', 'BIS-%s' % os.path.basename(self._filepath)),
                ('department', row_dict['Department']),
                ('update_frequency', row_dict['Update Frequency']),
                ('national_statistic', national_statistic),
                ('categories', row_dict['Categories']),
                ])),
            ])

        tags = schema_gov.TagSuggester.suggest_tags(pkg_dict)
        [tags.add(tag) for tag in schema_gov.tags_parse(row_dict['Tags'])]
        tags = list(tags)
        tags.sort()
        pkg_dict['tags'] = tags

        # snap to suggestions
        field_suggestions = [
            ['temporal_granularity', schema_gov.temporal_granularity_options],
            ['geographical_granularity', schema_gov.geographic_granularity_options],
            ['categories', schema_gov.category_options],
            ['department', schema_gov.government_depts],
            ]
        for field, suggestions in field_suggestions:
            val = pkg_dict['extras'][field]
            if val and val != '-' and val not in suggestions:
                suggestions_lower = [sugg.lower() for sugg in suggestions]
                if val.lower() in suggestions_lower:
                    val = suggestions[suggestions_lower.index(val.lower())]
                elif schema_gov.expand_abbreviations(val) in suggestions:
                    val = schema_gov.expand_abbreviations(val)
                elif val.lower() + 's' in suggestions:
                    val = val.lower() + 's'
                elif val.replace('&', 'and').strip() in suggestions:
                    val = val.replace('&', 'and').strip()
            if val and val != '-' and val not in suggestions:
                self.log("WARNING: Value for column '%s' of '%s' is not in suggestions '%s'" % (column, val, suggestions))
            pkg_dict['extras'][field] = val

        return pkg_dict
        
    @classmethod
    def name_munge(self, input_name):
        input_name = input_name.replace('-by-qualification-aim-mode-of-study-gender-and-', '-')
        return super(BisImporter, self).name_munge(input_name)



import os
import urllib
import datetime
import logging
import datetime

ONS_CACHE_PATH = os.path.abspath(os.path.expanduser('~/ons_data'))
if not os.path.exists(ONS_CACHE_PATH):
    os.makedirs(ONS_CACHE_PATH)
ONS_URL_BASE = 'http://www.statistics.gov.uk/hub/release-calendar/rss.xml?lday=%(lday)s&lmonth=%(lmonth)s&lyear=%(lyear)s&uday=%(uday)s&umonth=%(umonth)s&uyear=%(uyear)s'

MONTHS_PER_YEAR = 12
YEAR_ONS_DATA_STARTS = 2004

class OnsData(object):
    '''Manages download and parse of ONS data.'''
    @classmethod
    def download_recent(cls, ons_cache_dir=ONS_CACHE_PATH, log=False, days=7):
        ons = cls(ons_cache_dir, log)
        url, url_name = ons._get_url_recent(days=days)
        return ons.download(url, url_name, force_download=True)

    @classmethod
    def download_flexible(cls, ons_cache_dir=ONS_CACHE_PATH, log=False,
                          days=None, start_date=None, end_date=None):
        ons = cls(ons_cache_dir, log)
        url, url_name = ons._get_url_flexible(days=days,
                                              start_date=start_date,
                                              end_date=end_date)
        return ons.download(url, url_name, force_download=True)

    @classmethod
    def download_all(cls, ons_cache_dir=ONS_CACHE_PATH, log=False):
        ons = cls(ons_cache_dir, log)
        url_tuples = ons._get_urls_for_all_time()
        return ons.download_multiple(url_tuples)

    def __init__(self, local_cache_dir=ONS_CACHE_PATH, log=False):
        self._local_cache_dir = os.path.expanduser(local_cache_dir)
        self._url_base = ONS_URL_BASE
        self._logging = log
        
    def download_multiple(self, url_tuples):
        filepaths = []
        for url_tuple in url_tuples:
            filepaths.append(self.download(*url_tuple))
        return filepaths

    def download(self, url, url_name, force_download=False):
        local_filepath = os.path.join(self._local_cache_dir, 'ons_data_%s' % url_name)
        if force_download and os.path.exists(local_filepath):
            os.remove(local_filepath)
        if not os.path.exists(local_filepath):
            urllib.urlretrieve(url, local_filepath)
        else:
            self.log(logging.info, 'ONS Data already downloaded: %s' % url_name)
        return local_filepath
        
    def download_month(self, month, year):
        assert month <= MONTHS_PER_YEAR
        assert year > 2000
        url, url_name = _get_url_month(month, year)
        self.download(url, url_name)

    def _get_url(self, lday, lmonth, lyear, uday, umonth, uyear):
        params = { 'lday':lday, 'lmonth':lmonth, 'lyear':lyear,
                   'uday':uday, 'umonth':umonth, 'uyear':uyear, }
        url = self._url_base % params
        url_id = '%(lyear)s-%(lmonth)s-%(lday)s_-_%(uyear)s-%(umonth)s-%(uday)s' % params
        return url, url_id
    
    def _get_url_month(self, month, year):
        id = '%(lday)s-%(lmonth)s-%(lyear)s_-_%(uday)s-%(umonth)s-%(uyear)s'
        url = self._url_base % {'lday':1, 'lmonth':month, 'lyear':year,
                                'uday':31, 'umonth':month, 'uyear':year,}
        url_id = datetime.date(year, month, 1).strftime('%Y-%m')
        if year == self._get_today().year and \
               month == self._get_today().month:
            url_id += '_incomplete'
        return [url, url_id]
    
    def _get_url_recent(self, days=7):
        end_date = self._get_today()
        return self._get_url_flexible(days=days, end_date=end_date)

    def _get_url_flexible(self, days=None, start_date=None, end_date=None):
        '''If you specify just days then it defaults end_date to be today.
        Otherwise you need to specify exactly two parameters.'''
        if start_date:
            assert isinstance(start_date, datetime.date)
        if end_date:
            assert isinstance(end_date, datetime.date)
        assert (days and not (start_date or end_date)) or \
               sum((bool(days), bool(start_date), bool(end_date))) == 2, \
               (days, start_date, end_date)

        url_id = None
        if days and not (start_date or end_date):
            end_date = self._get_today()
        if not start_date:
            start_date = end_date - datetime.timedelta(days=days)
            url_id = end_date.strftime(str(days) + '_days_to_%Y-%m-%d')
        if not end_date:
            end_date = start_date + datetime.timedelta(days=days)            
            url_id = start_date.strftime(str(days) + '_days_from_%Y-%m-%d')
        url, basic_url_id = self._get_url_days_period(start_date, end_date)
        return url, url_id or basic_url_id
        
    def _get_url_days_period(self, start_date, end_date):
        assert isinstance(start_date, datetime.date)
        assert isinstance(end_date, datetime.date)
        period = end_date - start_date
        if period > datetime.timedelta(days=31):
            self.log(logging.warning, 'Period is longer than a month - you may find the ONS Hub truncates the results unexpectedly.')
        url = self._url_base % {'lday':start_date.strftime('%d'),
                                'lmonth':start_date.strftime('%m'),
                                'lyear':start_date.strftime('%Y'),
                                'uday':end_date.strftime('%d'),
                                'umonth':end_date.strftime('%m'),
                                'uyear':end_date.strftime('%Y'),
                                }
        url_id = '%s_to_%s' % (start_date.strftime('%Y-%m-%d'),
                                end_date.strftime('%Y-%m-%d'))
        return url, url_id

    def _get_urls_for_all_time(self):
        url_tuples = []
        this_year = self._get_today().year
        for year in range(YEAR_ONS_DATA_STARTS, this_year): # not including this year
            for month in range(1, MONTHS_PER_YEAR+1): # i.e. 1-12 inclusive
                url_tuples.append(self._get_url_month(month, year) + [False])
        this_month = self._get_today().month
        for month in range(1, this_month):
            url_tuples.append(self._get_url_month(month, this_year) + [False])
        url_tuples.append(self._get_url_month(this_month, this_year) + [True])
        return url_tuples

    def _get_today(self):
        return datetime.date.today()

    def log(self, log_func, msg):
        if self._logging:
            log_func(msg)
        else:
            print '%s: %s' % (log_func.func_name, msg)



from mass_changer import *
from common import ScriptError

class ChangeLicenses(object):
    def __init__(self, ckanclient, dry_run=False, force=False):
        '''
        Changes licenses of packages.
        @param ckanclient: instance of ckanclient to make the changes
        @param license_id: id of the license to change packages to
        @param force: do not stop if there is an error with one package
        '''
        self.ckanclient = ckanclient
        self.dry_run = dry_run
        self.force = force

    def change_all_packages(self, license_id):
        instructions = [
            ChangeInstruction(AnyPackageMatcher(),
                              BasicPackageChanger('license_id', license_id))
            ]
        self.mass_changer = MassChanger(self.ckanclient,
                                        instructions,
                                        dry_run=self.dry_run,
                                        force=self.force)
        self.mass_changer.run()

    def change_oct_2010(self, license_id):
        instructions = [
            ChangeInstruction(
                [
                    BasicPackageMatcher('license_id', 'localauth-withrights'),
                    BasicPackageMatcher('name', 'spotlightonspend-transactions-download'),
                    BasicPackageMatcher('name', 'better-connected-2010-council-website-performance-survey-results'),
                    ],
                NoopPackageChanger()),
            ChangeInstruction(
                AnyPackageMatcher(),
                BasicPackageChanger('license_id', self.license_id)),
            ]
        self.mass_changer = MassChanger(self.ckanclient,
                                        instructions,
                                        dry_run=self.dry_run,
                                        force=self.force)
        self.mass_changer.run()



from sqlalchemy.util import OrderedDict
from pylons.i18n import _, ungettext, N_, gettext
from pylons import config

from ckan.forms import common
from ckan.forms import package
from ckan.forms import package_gov
from ckan.forms.builder import FormBuilder
from ckan.forms.common import ResourcesField, TagField, GroupSelectField, package_name_validator
from ckan import model
from ckanext.dgu import schema as schema_gov
from ckan.lib.helpers import literal

# Setup the fieldset
def build_package_gov_form_v3(is_admin=False, user_editable_groups=None,
                              statistics=False, inventory=False, **kwargs):
    assert not (statistics and inventory) # can't be both

    # Restrict fields
    restrict = str(kwargs.get('restrict', False)).lower() not in \
               ('0', 'no', 'false', 0, False)
    
    builder = FormBuilder(model.Package)

    # Extra fields
    builder.add_field(GroupSelectField('groups', allow_empty=True, user_editable_groups=user_editable_groups))
    builder.add_field(ResourcesField('resources', hidden_label=True, fields_required=set(['url', 'format'])))
    builder.add_field(TagField('tags'))
    builder.add_field(common.TextExtraField('external_reference'))
    builder.add_field(common.DateExtraField('date_released'))
    builder.add_field(common.DateExtraField('date_updated'))
    builder.add_field(common.DateExtraField('date_update_future'))
    builder.add_field(common.SuggestedTextExtraField('update_frequency', options=schema_gov.update_frequency_options))
    builder.add_field(common.SuggestedTextExtraField('geographic_granularity', options=schema_gov.geographic_granularity_options))
    builder.add_field(package_gov.GeoCoverageExtraField('geographic_coverage'))
    builder.add_field(common.SuggestedTextExtraField('temporal_granularity', options=schema_gov.temporal_granularity_options))
    builder.add_field(common.DateRangeExtraField('temporal_coverage'))
    builder.add_field(common.TextExtraField('precision'))
    builder.add_field(common.SuggestedTextExtraField('department', options=schema_gov.government_depts))
    builder.add_field(common.TextExtraField('agency'))
    builder.add_field(common.TextExtraField('taxonomy_url'))
    builder.add_field(common.TextExtraField('mandate'))
    builder.add_field(common.TextExtraField('publisher'))
    builder.add_field(common.CoreField('license_id', value='uk-ogl'))
#   TODO Remove National Statistic from core form, when we can choose the
#   form
#    if statistics:
    if True:
        builder.add_field(common.CheckboxExtraField('national_statistic'))
    if statistics:
        builder.add_field(common.SuggestedTextExtraField('series'))
    if inventory:
        builder.add_field(common.DateExtraField('date_disposal'))
        builder.add_field(common.DateExtraField('date_planned_publication'))
        builder.add_field(common.SuggestedTextExtraField('permanent_preservation'), options=schema_gov.yes_no_not_yet_options)
        builder.add_field(common.CheckboxExtraField('disclosure_foi'))
        builder.add_field(common.CheckboxExtraField('disclosure_eir'))
        builder.add_field(common.CheckboxExtraField('disclosure_dpa'))

    # Labels and instructions
    builder.set_field_text('title', instructions='The title of the data set.', further_instructions='The main subject of the data should be included with a lay term. For cross-government data requirements, such as spend data specify the public body the data belongs to or its geographical coverage, in order to distinguish your data from other similar datasets in data.gov.uk. If the data relates to a period of time, include that in the name, although this would not be appropriate for data which is updated over time. It is not a description - save that for the Abstract element. Do not give a trailing full stop.', hints=literal('e.g. Payments to suppliers with a value over &pound;500 from Harlow Council'))
    builder.set_field_text('name', 'Identifier', instructions='A public unique identifier for the dataset', further_instructions='It should be roughly readable, with dashes separating words.', hints='Format: Two or more lowercase alphanumeric, dash (-) or underscore (_) characters. e.g. uk-road-traffic-statistics-2008 or local-authority-spend-over-500-harlow')
    builder.set_field_text('notes', 'Abstract', instructions='The main description of the dataset', further_instructions='It is often displayed with the package title. In particular, it should start with a short sentence that describes the data set succinctly, because the first few words alone may be used in some views of the data sets. Here is the place to state if there are any limitations or deficiencies to the data in order to enable users to evaluate the information; even incomplete data may be adequate for some users.')
    builder.set_field_text('date_released', instructions='The date of the official release of the initial version of the dataset', further_instructions='This is probably not the date that it is uploaded to data.gov.uk. Be careful not to confuse a new \'version\' of some data with a new dataset covering another time period or geographic area.', hints='DD/MM/YYYY')
    builder.set_field_text('date_updated', instructions='The date of release of the most recent version of the dataset', further_instructions='This is not necessarily the date when it was updated on data.gov.uk. As with \'Date released\', this is for updates to a particular dataset, such as corrections or refinements, not for that of a new time period.', hints='DD/MM/YYYY')
    builder.set_field_text('date_update_future', 'Date to be published', instructions='When the dataset will be updated in the future, if appropriate', hints='DD/MM/YYYY')
    builder.set_field_text('update_frequency', instructions='How frequently the dataset is updated with new versions', further_instructions='For one-off data, use \'never\'. For those once updated but now discontinued, use \'discontinued\'.')
    builder.set_field_text('precision', instructions='Indicate the level of precision in the data, to avoid over-interpretation.', hints="e.g. 'per cent to two decimal places' or 'as supplied by respondents'")
    builder.set_field_text('geographic_granularity', instructions='The lowest level of geographic detail', further_instructions="This should give the lowest level of geographic detail given in the dataset if it is aggregated. If the data is not aggregated, and so the dataset goes down to the level of the entities being reported on (such as school, hospital, or police station), use 'point'. If none of the choices is appropriate or the granularity varies, please specify in the 'other' element.")
    builder.set_field_text('geographic_coverage', instructions='The geographic coverage of this dataset', further_instructions='Where a dataset covers multiple areas, the system will automatically group these (e.g. \'England\', \'Scotland\' and \'Wales\' all being \'Yes\' would be shown as \'Great Britain\').')
    builder.set_field_text('temporal_granularity', instructions='The lowest level of temporal detail granularity', further_instructions="This should give the lowest level of temporal detail given in the dataset if it is aggregated, expressed as an interval of time. If the data is not aggregated over time, and so the dataset goes down to the instants that reported events occurred (such as the timings of high and low tides), use 'point'. If none of the choices is appropriate or the granularity varies, please specify in the 'other' element.")
    builder.set_field_text('temporal_coverage', instructions='The temporal coverage of this dataset.', further_instructions='If available, please indicate the time as well as the date. Where data covers only a single day, the \'To\' sub-element can be left blank.', hints='e.g. 21/03/2007 - 03/10/2009 or 07:45 31/03/2006')
    builder.set_field_text('url', instructions='The Internet link to a web page discussing the dataset.', hints='e.g. http://www.somedept.gov.uk/growth-figures.html')
    builder.set_field_text('taxonomy_url', instructions='An Internet link to a web page describing the taxonomies used in the dataset, if any, to ensure they understand any terms used.', hints='e.g. http://www.somedept.gov.uk/growth-figures-technical-details.html')
    builder.set_field_text('department', instructions='The Department under which the dataset is collected and published', further_instructions='Note, this department is not necessarily directly undertaking the collection/publication itself - use the Agency element where this applies.')
    builder.set_field_text('agency', instructions='The agency or arms-length body responsible for the data collection', further_instructions='Please use the full title of the body without any abbreviations, so that all items from it appear together. The data.gov.uk system will automatically capture this where appropriate.', hints='e.g. Environment Agency')
    builder.set_field_text('publisher', instructions='The public body credited with the publication of this data', further_instructions="An 'over-ride' for the system, to determine the correct public body to credit, when it might not be clear if this is the Agency or Department. This could be used where the public branding of the work of an agency is as its parent department.")
    builder.set_field_text('author', 'Contact', instructions='The permanent contact point for the public to enquire about this particular dataset. In addition, the Public Data and Transparency Team will use it for any suggestions for changes, feedback, reports of mistakes in the datasets or metadata.', further_instructions='This should be the name of the section of the agency or Department responsible, and should not be a named person. Particular care should be taken in choosing this element.', hints='Examples: Statistics team, Public consultation unit, FOI contact point')
    builder.set_field_text('author_email', 'Contact email', instructions='A generic official e-mail address for members of the public to contact, to match the \'Contact\' element.', further_instructions='A new e-mail address may need to be created for this function.')
#    if statistics:
    if True:
        builder.set_field_text('national_statistic', 'National Statistic', instructions='Indicate if the dataset is a National Statistic', further_instructions='This is so that it can be highlighted.')
    if statistics:
        builder.set_field_text('series', instructions='The name of a series or collection that this data is part of.', further_instructions='This is needed for National Statistics. For example \'Wages Weekly Index\'.')
    if inventory:
        builder.set_field_text('date_disposal', instructions='Date of removal of the data from the public body\'s systems.', further_instructions='This is a future date when it is intended to remove the data, eventually replaced with the actual date the disposal was implemented (if appropriate).', hints='DD/MM/YYYY')
        builder.set_field_text('date_planned_publication', 'Planned publication date', instructions='To be used when adding data intended for future publication to the Inventory', hints='DD/MM/YYYY')
        builder.set_field_text('permanent_preservation', 'Selected for permanent preservation', instructions='Data, whose sensitivity currently prevents publication, selected by the National Archives as worthy of permanent preservation.  (It is intended that published data will automatically be preserved in the UK Government Web Archive.)')
        builder.set_field_text('disclosure_foi', instructions='Indicates whether or not data is exempt from publication by virtue of the Freedom of Information Act 2000.')
        builder.set_field_text('disclosure_eir', instructions='Indicates wheter or not data is exempt from publication by virtue of the Environmental Information Regulations 2004.')
        builder.set_field_text('disclosure_dpa', instructions='Indicates whether or not data is exempt from publication by virtue of  the Data Protection Act 1998, taking into account that personal data whilst not of itself disclosive, may, if combined with other data in the public domain, become so.')
    builder.set_field_text('mandate', instructions='An Internet link to the enabling legislation that serves as the mandate for the collection or creation of this data, if appropriate.', further_instructions='This should be taken from The National Archives\' Legislation website, and where possible be a link directly to the relevant section of the Act.', hints='For example Public Record Act s.2 would be: http://www.legislation.gov.uk/id/ukpga/Eliz2/6-7/51/section/2')
    builder.set_field_text('license_id', 'Licence', instructions='The licence under which the dataset is released.', further_instructions=literal('For most situations of central Departments\' and Local Authority data, this should be the \'Open Government Licence\'. If you wish to release the data under a different licence, please contact the <a href="mailto:PublicData@nationalarchives.gsi.gov.uk">Public Data and Transparency Team</a>.'))
    builder.set_field_text('resources', instructions='The files containing the data or address of the APIs for accessing it', further_instructions=literal('These can be repeated as required. For example if the data is being supplied in multiple formats, or split into different areas or time periods, each file is a different \'resource\' which should be described differently. They will all appear on the dataset page on data.gov.uk together.<br/> <b>URL:</b> This is the Internet link directly to the data - by selecting this link in a web browser, the user will immediately download the full data set. Note that datasets are not hosted by data.gov.uk, but by the responsible department<br/> e.g. http://www.somedept.gov.uk/growth-figures-2009.csv<br/><b>Format:</b> This should give the file format in which the data is supplied. You may supply the data in a form not listed here, constrained by the <a href="http://data.gov.uk/blog/new-public-sector-transparency-board-and-public-data-transparency-principles" target="_blank">Public Sector Transparency Board\'s principles</a> that require that all data is available in an \'open and standardised format\' that can be read by a machine. Data can also be released in formats that are not machine-processable (e.g. PDF) alongside this.<br/>'), hints='Format choices: CSV | RDF | XML | XBRL | SDMX | HTML+RDFa | Other as appropriate')
    builder.set_field_text('tags', instructions='Tags can be thought of as the way that the packages are categorised, so are of primary importance.', further_instructions=literal('One or more tags should be added to give the government department and geographic location the data covers, as well as general descriptive words. The <a href="http://www.esd.org.uk/standards/ipsv_abridged/" target="_blank">Integrated Public Sector Vocabulary</a> may be helpful in forming these.'), hints='Format: Two or more lowercase alphanumeric or dash (-) characters; different tags separated by spaces. As tags cannot contain spaces, use dashes instead. e.g. for a dataset containing statistics on burns to the arms in the UK in 2009: nhs uk arm burns medical-statistics')
    # Options/settings
    builder.set_field_option('name', 'validate', package_name_validator)
    builder.set_field_option('license_id', 'dropdown', {'options':[('', None)] + model.Package.get_license_options()})
    builder.set_field_option('state', 'dropdown', {'options':model.State.all})
    builder.set_field_option('notes', 'textarea', {'size':'60x15'})
    builder.set_field_option('title', 'required')
    builder.set_field_option('notes', 'required')
    builder.set_field_option('department', 'required')
    builder.set_field_option('license_id', 'required')
    builder.set_field_option('tags', 'with_renderer', package_gov.SuggestTagRenderer)

    if restrict:
        for field_name in ('name', 'department', 'national_statistic'):
            builder.set_field_option(field_name, 'readonly', True)
    
    # Layout
    field_groups = OrderedDict([
        (_('Basic information'), ['title', 'name',
                                  'notes']),
        (_('Details'), ['date_released', 'date_updated', 'date_update_future',
                        'update_frequency',
                        'precision', 
                        'geographic_granularity', 'geographic_coverage',
                        'temporal_granularity', 'temporal_coverage',
                        'url', 'taxonomy_url']),
        (_('Resources'), ['resources']),
        (_('More details'), ['department', 'agency',
                             'author', 'author_email',
                             'mandate', 'license_id',
                             'tags']),
        ])
#    if statistics:
    field_groups['More details'].append('national_statistic')
    if statistics:
        field_groups['More details'].append('series')
    if inventory:
        field_groups['Details'].insert('date_planned_publication', 0)
        field_groups['Details'].insert('date_disposal', 4)
        field_groups['Details'].insert('permanent_preservation', 5)
        field_groups['More details'].insert('disclosure_foi', 6)
        field_groups['More details'].insert('disclosure_eir', 7)
        field_groups['More details'].insert('disclosure_dpa', 8)
    if is_admin:
        field_groups['More details'].append('state')
    builder.set_label_prettifier(package.prettify)
    builder.set_displayed_fields(field_groups)
    return builder
    # Strings for i18n:
    # (none - not translated at the moment)

def get_gov3_fieldset(is_admin=False, user_editable_groups=None, **kwargs):
    '''Returns the standard fieldset
    '''
    return build_package_gov_form_v3(is_admin=is_admin, user_editable_groups=user_editable_groups, **kwargs).get_fieldset()



import os
import datetime

from sqlalchemy.util import OrderedDict
from pylons import config
from nose.tools import assert_equal

from ckan.tests import *
import ckan.model as model
from ckan.lib import spreadsheet_importer
from ckan.tests.wsgi_ckanclient import WsgiCkanClient
from ckan.tests.lib.test_spreadsheet_importer import CKAN_SRC_DIR, CSV_EXTENSION, XL_EXTENSION, SPREADSHEET_DATA_MAP, ExampleFiles
from ckanext.dgu.tests import PackageDictUtil
from ckanext.dgu.cospread.cospread import MultipleCospreadDataRecords, CospreadImporter

TEST_DIR = os.path.dirname(os.path.abspath(__file__))
EXAMPLES_DIR = os.path.join(TEST_DIR, 'samples')
EXAMPLE_FILEBASE = 'cospread'

examples = ExampleFiles(EXAMPLES_DIR, EXAMPLE_FILEBASE)

# column titles for older pro forma; used in example 1
expected_titles_1 = set((
    "Package name",
    "Title",
    "CO Identifier",
    "Notes",
    "Date released",
    "Date updated",
    "Update frequency",
    "Geographical Granularity - Standard", "Geographical Granularity - Other",
    "Geographic coverage - England","Geographic coverage - N. Ireland","Geographic coverage - Scotland", "Geographic coverage - Wales","Geographic coverage - Overseas","Geographic coverage - Global",
    "Temporal Granularity - Standard","Temporal Granularity - Other",
    "File format",
    "Categories",
    "National Statistic",
    "Precision",
    "URL",
    "Download URL",
    "Taxonomy URL",
    "Department",
    "Agency responsible",
    "Contact - Permanent contact point","Contact - E-mail address.",
    "Maintainer - ", "Maintainer - E-mail address",
    "Licence", "Tags"
    ))
# titles for more recent cospread pro forma; used for clg example
expected_titles_2 = set((
    "Package name",
    "Title",
    "CO Reference",
    "Notes",
    "Date released",
    "Date updated",
    "Update frequency",
    "Geographical Granularity - Standard", "Geographical Granularity - Other",
    "Geographic coverage - England","Geographic coverage - N. Ireland","Geographic coverage - Scotland", "Geographic coverage - Wales","Geographic coverage - Overseas","Geographic coverage - Global",
    "Temporal Granularity - Standard","Temporal Granularity - Other",
    "Temporal Coverage - To\n(if needed)", "Temporal Coverage - From",
    "Download file format",
    "Download Description",
    "Precision",
    "URL",
    "Download URL",
    "Taxonomy URL",
    "Department",
    "Agency responsible",
    "Author - Permanent contact point for members of the public; not an individual.", "Author - E-mail address.",
    "Maintainer - Blank unless not the author.", "Maintainer - E-mail address, if needed.",
    "Licence", "Tags"
    ))
    
example_record = OrderedDict([
    ('Package name', 'child-protection-plan-england-2009'),
    ('Title', 'Child Protection Plan'),
    ('CO Identifier', 'DCSF-DCSF-0017'),
    ('Notes', 'Referrals, assessment and children and young people who are the subjects of child protection plans (on the child protection register) for year ending March 2009'),
    ('Date released', '17/09/2009'),
    ('Date updated', '17/09/09'),
    ('Update frequency', 'Annually'),
    ('Geographical Granularity', 'Local Authority'),
    ('Geographic coverage - England', 'Yes'),
    ('Geographic coverage - N. Ireland', 'No'),
    ('Geographic coverage - Scotland', 'No'),
    ('Geographic coverage - Wales', 'No'),
    ('Geographic coverage - Overseas', 'No'),
    ('Geographic coverage - Global', 'No'),
    ('Temporal Granularity', 'Years'),
    ('resources', [{'File format': 'XLS',
                    'Download URL': 'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000873/FINALAdditionalTables1to13.xls\n\nhttp://www.dcsf.gov.uk/rsgateway/DB/SFR/s000873/NationalIndicatorTables.xls'}]),
    ('Categories', 'Health, well-being and Care'),
    ('National Statistic', 'Yes'),
    ('Precision', 'Numbers rounded to nearest 100 if over 1,000, and to the nearest 10 otherwise.  Percentage to nearest whole number.'),
    ('URL', 'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000873/index.shtml'),
    ('Taxonomy URL', ''),
    ('Department', 'Department for Education'),
    ('Agency responsible', ''),
    ('Contact - Permanent contact point', 'DCSF Data Services Group'),
    ('Contact - E-mail address.', 'statistics@dcsf.gsi.gov.uk'),
    ('Maintainer - ', ''),
    ('Maintainer - E-mail address', ''),
    ('Licence', 'UK Crown Copyright'),
    ('Tags', 'dcsf england child-protection-plan-statistics referrals assessments child-protection-register'),
    ])
example_pkg_dict = OrderedDict([
    ('name', u'child-protection-plan-england-2009'),
    ('title', u'Child Protection Plan'),
    ('version', None),
    ('url', u'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000873/index.shtml'),
    ('author', u'DCSF Data Services Group'),
    ('author_email', u'statistics@dcsf.gsi.gov.uk'),
    ('maintainer', u''),
    ('maintainer_email', u''),
    ('notes', u'Referrals, assessment and children and young people who are the subjects of child protection plans (on the child protection register) for year ending March 2009'),
    ('license_id', u'uk-ogl'),
    ('tags', ['child', 'child-protection', 'children']), 
    ('groups', ['ukgov']),
    ('resources', [OrderedDict([
        ('url', 'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000873/FINALAdditionalTables1to13.xls'),
        ('format', 'XLS'),
        ('description', ''),
        ]),
                   OrderedDict([
        ('url', 'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000873/NationalIndicatorTables.xls'),
        ('format', 'XLS'),
        ('description', ''),
                       ]),
                   ]),
    ('extras', OrderedDict([
        ('external_reference', 'DCSF-DCSF-0017'),
        ('date_released', '2009-09-17'),
        ('date_updated', '2009-09-17'),
        ('temporal_granularity', 'years'),
        ('temporal_coverage_to', ''),
        ('temporal_coverage_from', ''),
        ('geographic_coverage', '100000: England'),
        ('geographical_granularity', 'local authority'),
        ('agency', u''),
        ('precision', 'Numbers rounded to nearest 100 if over 1,000, and to the nearest 10 otherwise.  Percentage to nearest whole number.'),
        ('taxonomy_url', ''),
        ('import_source', 'COSPREAD-cospread1.csv'),
        ('department', u'Department for Education'),
        ('update_frequency', 'Annually'),
        ('national_statistic', ''),
        ('categories', 'Health, well-being and Care'),
        ])
     ),
    ])

clg_record = OrderedDict([
    ('Package name', 'Total-number-of-dwellings-owned-by-your-local-authority'),
    ('Title', 'Total number of dwellings owned by your local authority'),
    ('CO Identifier', ''),
    ('Notes', 'The purpose % Non-Decent Homes.\n\nThe links below provide access to the Local Data Exchange (LDEx) API which can be used to discover data about this indicator.'),
    ('Date released', datetime.date(2002, 4, 1)),
    ('Date updated', ''),
    ('Update frequency', ''),
    ('Geographical Granularity', 'Local Authority (District)'),
    ('Geographic coverage - England', 'England'),
    ('Geographic coverage - N. Ireland', ''),
    ('Geographic coverage - Scotland', ''),
    ('Geographic coverage - Wales', ''),
    ('Geographic coverage - Overseas', ''),
    ('Geographic coverage - Global', ''),
    ('Temporal Granularity', ''),
    ('Temporal Coverage - From', '2002/03'),
    ('Temporal Coverage - To\n(if needed)', '2008/09'),
    ('resources', [{'File format': 'RDF',
                    'Download URL': 'http://ldexincubator.communities.gov.uk/service/ldex/housingandplanning/api/housingandplanning-indicator/A_TOAlDW/doc.rdf',
                    'Download Description': 'RDF Description',
                    },
                   {'File format': 'RDF',
                    'Download URL': 'http://doc2.rdf',
                    'Download Description': 'File 2',
                    }]),
    ('Precision', ''),
    ('URL', 'http://ldexincubator.communities.gov.uk/service/ldex/housingandplanning/doc/housingandplanning-indicator/A_TOAlDW/doc.html'),
    ('Taxonomy URL', ''),
    ('Department', 'Department for Communities and Local Government'),
    ('Agency responsible', ''),
    ('Contact - Permanent contact point', ''),
    ('Contact - E-mail address.', 'open_data@yahoo.com'),
    ('Maintainer - ', ''),
    ('Maintainer - E-mail address', ''),
    ('Licence', 'UK Crown Copyright with data.gov.uk rights'),
    ('Tags', 'Housing-and-Planning-View-Indicator-Places LDEx LDEX Local-Data-Exchange Department-for-Communities-and-Local-Government Communities Local-Government Housing-and-Planning-Indicator housing planning'),
    ])

clg_pkg_dict = OrderedDict([
    ('name', u'total-number-of-dwellings-owned-by-your-local-authority'),
    ('title', u'Total number of dwellings owned by your local authority'),
    ('version', None),
    ('url', u'http://ldexincubator.communities.gov.uk/service/ldex/housingandplanning/doc/housingandplanning-indicator/A_TOAlDW/doc.html'),
    ('author', u''),
    ('author_email', u'open_data@yahoo.com'),
    ('maintainer', u''),
    ('maintainer_email', u''),
    ('notes', u'The purpose % Non-Decent Homes.\n\nThe links below provide access to the Local Data Exchange (LDEx) API which can be used to discover data about this indicator.'),
    ('license_id', u'uk-ogl'),
    ('tags', ['communities', 'department-for-communities-and-local-government', 'housing', 'housing-and-planning-indicator', 'housing-and-planning-view-indicator-places', 'ldex', 'local-authority', 'local-data-exchange', 'local-government', 'planning']),
    ('groups', ['ukgov']),
    ('resources', [OrderedDict([
        ('url', 'http://ldexincubator.communities.gov.uk/service/ldex/housingandplanning/api/housingandplanning-indicator/A_TOAlDW/doc.rdf'),
        ('format', 'RDF'),
        ('description', 'RDF Description'),
        ]),
                   OrderedDict([
        ('url', 'http://doc2.rdf'),
        ('format', 'RDF'),
        ('description', 'File 2')
                       ]),
                   ]),
    ('extras', OrderedDict([
        ('external_reference', ''),
        ('date_released', '2002-04-01'),
        ('date_updated', ''),
        ('temporal_granularity', ''),
        ('temporal_coverage_to', '2009'),
        ('temporal_coverage_from', '2002'),
        ('geographic_coverage', '100000: England'),
        ('geographical_granularity', 'Local Authority (District)'),
        ('agency', u''),
        ('precision', ''),
        ('taxonomy_url', ''),
        ('import_source', 'COSPREAD-cospread-clg.xls'),
        ('department', u'Department for Communities and Local Government'),
        ('update_frequency', ''),
        ('national_statistic', ''),
        ('categories', ''),
        ])
     ),
    ])

class TestCospreadDataRecords:
    @classmethod
    def setup_class(self):
        self.data = examples.get_data('1', CSV_EXTENSION)
        self.data_records = MultipleCospreadDataRecords(self.data)
        
    def test_0_title_row(self):
        assert self.data_records.records_list[0].titles[0] == u'Package name', \
               self.data_records.records_list[0].titles

    def test_1_titles(self):
        titles = set(self.data_records.records_list[0].titles)
        title_difference = expected_titles_1 ^ titles
        assert not title_difference, title_difference

    def test_2_records(self):
        self.records = [record for record in self.data_records.records]
        assert len(self.records) == 3, self.records
        for key, value in example_record.items():
            assert self.records[0].has_key(key), 'Expected key %r in record: %r' % (key, self.records[0].keys())
            assert_equal(self.records[0][key], value)
        expected_keys = set([key for key, value in example_record.items()])
        keys = set(self.records[0].keys())
        key_difference = expected_keys - keys
        assert not key_difference, key_difference


class TestCospreadDataRecordsClg:
    @classmethod
    def setup_class(self):
        self.data = examples.get_data('-clg', XL_EXTENSION)
        self.data_records = MultipleCospreadDataRecords(self.data)
        
    def test_0_title_row(self):
        assert self.data_records.records_list[0].titles[0] == u'Package name', \
               self.data_records.records_list[0].titles

    def test_1_titles(self):
        titles = set(self.data_records.records_list[0].titles)
        title_difference = expected_titles_2 ^ titles
        if title_difference:
            msg = 'Titles should not have: %r' % (title_difference & titles)
            msg += '\nTitles expected but not received: %r' % (title_difference & expected_titles_2)
        assert not title_difference, msg

    def test_1_multiple_sheets(self):
        assert len(self.data_records.records_list) == 3, self.data_records.records_list

    def test_2_records(self):
        self.records = [record for record in self.data_records.records]
        assert len(self.records) == 3, '%i, %r' % (len(self.records),
                                                   self.records)
        # check record titles
        assert_equal(self.records[0]['Title'], u'Total number of dwellings owned by your local authority')
        assert_equal(self.records[1]['Title'], u'CWI Children in Need Domain')
        assert_equal(self.records[2]['Title'], u'NI 001 - Percentage of people who believe people from different backgrounds get on well together in their local area')
        
        # check first record thoroughly
        for key, value in clg_record.items():
            assert self.records[0].has_key(key), 'Expected key %r in record: %r' % (key, self.records[0].keys())
            assert_equal(self.records[0][key] or None, value or None)
        expected_keys = set([key for key, value in clg_record.items()])
        keys = set(self.records[0].keys())
        key_difference = expected_keys - keys
        assert not key_difference, key_difference


class TestImport:
    @classmethod
    def setup_class(self):
        self._filepath = examples.get_spreadsheet_filepath('1', CSV_EXTENSION)
        self.importer = CospreadImporter(include_given_tags=False, filepath=self._filepath)
        self.pkg_dicts = [pkg_dict for pkg_dict in self.importer.pkg_dict()]

    def test_0_name_munge(self):
        def test_name_munge(name, expected_munge):
            munge = self.importer.name_munge(name)
            assert munge == expected_munge, 'Got %s not %s' % (munge, expected_munge)
        test_name_munge('hesa-(1994-1995)', 'hesa-1994-1995')

        # 96 characters should be left alone
        test_name_munge('ni-198q-children-travelling-to-school-mode-of-transport-usually-used-pupils-aged-5-16-by-cycling', 'ni-198q-children-travelling-to-school-mode-of-transport-usually-used-pupils-aged-5-16-by-cycling')
        test_name_munge('a'*105, 'a'*100)

    def test_0_munge(self):
        def test_munge(name, expected_munge):
            munge = self.importer.munge(name)
            assert munge == expected_munge, 'Got %s not %s' % (munge, expected_munge)        
        test_munge('a$b cD:e f-g%  h', 'ab-cd-e-f-g-h')

    def test_1_record_2_package(self):
        pkg_dict = self.importer.record_2_package(example_record)

        log = self.importer.get_log()
        assert log[0].startswith("WARNING: Value for column 'Categories' of 'Health, well-being and Care' is not in suggestions '["), log[0]
        assert log[1].startswith("WARNING: URL doesn't start with http: test.html"), log[1]
        assert log[2].startswith("WARNING: URL doesn't start with http: test.json"), log[2]
        assert log[3].startswith("WARNING: Value for column 'Categories' of 'Health, well-being and Care' is not in suggestions '["), log[3]
        assert_equal(len(log), 4, log)

        PackageDictUtil.check_dict(pkg_dict, example_pkg_dict)
        expected_keys = set([key for key, value in example_pkg_dict.items()])
        keys = set(pkg_dict.keys())
        key_difference = expected_keys - keys
        assert not key_difference, key_difference

    def test_2_overall_import(self):
        pkg_dict = self.importer.record_2_package(example_record)
        assert_equal(self.pkg_dicts[0], pkg_dict)


class TestImportClg():
    @classmethod
    def setup_class(self):
        self._filepath = examples.get_spreadsheet_filepath('-clg', XL_EXTENSION)
        self.importer = CospreadImporter(include_given_tags=True, filepath=self._filepath)
        self.pkg_dicts = [pkg_dict for pkg_dict in self.importer.pkg_dict()]

    def test_all_sheets_found(self):
        assert len(self.pkg_dicts) == 3

    def test_include_given_tags(self):
        assert 'housing-and-planning-view-indicator-places' in self.pkg_dicts[0]['tags'], self.pkg_dicts[0]['tags']

    def test_1_record_2_package(self):
        pkg_dict = self.importer.record_2_package(clg_record)

        log = self.importer.get_log()
        assert log[0].startswith("WARNING: Value for column 'Geographical Granularity' of 'Local Authority (District)'"), log[0]
        assert log[1].startswith("WARNING: Value for column 'Geographical Granularity' of 'Super Output Area'"), log[1]
        assert log[2].startswith("WARNING: Value for column 'Geographical Granularity' of 'Local Authority District (LAD), County/Unitary Authority, Government Office Region (GOR), National'"), log[2]
        assert log[3].startswith("WARNING: Value for column 'Geographical Granularity' of 'Local Authority (District)' is not in suggestions"), log[3]
        assert_equal(len(log), 4, log)

        PackageDictUtil.check_dict(pkg_dict, clg_pkg_dict)
        expected_keys = set([key for key, value in clg_pkg_dict.items()])
        keys = set(pkg_dict.keys())
        key_difference = expected_keys - keys
        assert not key_difference, key_difference

    def test_2_overall_import(self):
        pkg_dict = self.importer.record_2_package(clg_record)
        assert_equal(self.pkg_dicts[0]['name'], pkg_dict['name'])



'''
Takes a list of package names.
For each package:
 * double-checks it has a URL but no resources
 * creates a new resource with download url according to the resource
'''
from mass_changer import *
from common import ScriptError

log = __import__("logging").getLogger(__name__)

pkg_name_list = '''anti-social-behaviour-orders-1999-2007
asylum-applications-jan-mar-2009
control-of-immigration-quarterly-statistical-summary-united-kingdom-2009-october-december
coroners-statistics-england-and-wales
courts-statistics-user-survey-england-and-wales
court-statistics-company-insolvency-and-bankruptcy-england-and-wales
court-statistics-england-and-wales
court-statistics-mortages-and-landlord-possession-england-and-wales
crime-in-england-and-wales
crime-statistics-local-reoffending-england-and-wales
crime-statistics-prison-and-probation-england-and-wales
crime-statistics-reoffending-of-adults-england-and-wales
crime-statistics-reoffending-of-juvenilles-england-and-wales
data_gov_uk-datasets
digest-uk-energy-statistics-2008
directgov-central-hottest-pages-monthly
directgov-central-internal-search-terms-monthly
directgov-section-visits-monthly
electricity-consumption-2007
electricity-gas-consumption-2007
energy-consumption-uk-2008
final-energy-consumption-2007
foi-statistics-uk-central-government
fuel-poverty-statistics-2007
gas-consumption-2007
gb-reported-bicycling-accidents
gb-road-traffic-counts
gb-traffic-matrix
greenhouse-gas-emissions-2008
high-level-indicators-energy-use-2006
judicial-and-court-statistics-england-and-wales
laboratory-tests-and-prices
local-authority-carbon-dioxide-emissions-2007
magistrates-courts-statistics-survey-england-and-wales
monthly-energy-prices
monthly-energy-trends
ni_012_refused_and_deferred_houses_in_multiple_occupation_hmos_licence_applications_leading_to_immig
ni_013_migrants_english_language_skills_and_knowledge
ni_023_perceptions_that_people_in_the_area_treat_one_another_with_respect_and_consideration
ni_024_satisfaction_with_the_way_the_police_and_local_council_dealt_with_anti-social_behaviour
ni_025_satisfaction_of_different_groups_with_the_way_the_police_and_local_council_dealt_with_anti-so
ni_026_specialist_support_to_victims_of_a_serious_sexual_offence
ni_029_gun_crime_rate
ni_031_re-offending_rate_of_registered_sex_offenders
ni_032_repeat_incidents_of_domestic_violence
ni_034_domestic_violence_-_murder
ni_036_protection_against_terrorist_attack
ni_038_drug_related_class_a_offending_rate
ni_078_reduction_in_number_of_schools_where_fewer_than_30_of_pupils_achieve_5_or_more_a-_c_grades_at
ni_101_looked_after_children_achieving_5_a-c_gcses_or_equivalent_at_key_stage_4_including_english_an
ni_109_delivery_of_sure_start_childrens_centres
ni_126_early_access_for_women_to_maternity_services
ni_127_self_reported_experience_of_social_care_users
ni_128_user_reported_measure_of_respect_and_dignity_in_their_treatment
ni_181_time_taken_to_process_housing_benefit-council_tax_benefit_new_claims_and_change_events
ni_184_food_establishments_in_the_area_which_are_broadly_compliant_with_food_hygiene_law
ni_185_co2_reduction_from_local_authority_operations
ni_190_achievement_in_meeting_standards_for_the_control_system_for_animal_health
ni_194_air_quality_-_reduction_in_nox_and_primary_pm10_emissions_through_local_authorities_estate_an
other-fuels-consumption-2006
police-use-firearms-england-wales-2007-2008
prison-end-of-custody-licence-releases-and-recalls-england-and-wales
prison-population-england-and-wales
probation-offender-management-caseload-statistics-england-and-wales
probation-statistics-quarterly-brief-england-and-wales
quality-indicators-energy-data-2007
quarterly-energy-prices
quarterly-energy-trends
road-transport-energy-consumption-2007
sentencing-statistics-england-and-wales
statistics-terrorism-arrests-outcomes-2001-2008
ukba-control-of-immigration-statistics-2008
ukba-control-of-immigration-statistics-2008-supplementary-tables
uk-energy-in-brief-2008
uk-energy-sector-indicators-background-2008
uk-energy-sector-indicators-key-supporting-2008
uk-exportcontrollists
uk-exportcontrol-sanctions
uk-export-control-statistics
uk-glossary-exportcontrol
uk-ipo-offences
weekly-fuel-prices
'''.split()
pkg_name_list = [name for name in pkg_name_list if name]

class TransferUrl(object):
    def __init__(self, ckanclient, dry_run=False, force=False):
        '''
        Changes licenses of packages.
        @param ckanclient: instance of ckanclient to make the changes
        @param license_id: id of the license to change packages to
        @param force: do not stop if there is an error with one package
        '''
        self.ckanclient = ckanclient
        self.dry_run = dry_run
        self.force = force

    def transfer_url(self):
        instructions = [
            ChangeInstruction(
                [
                    TransferUrlMatcher(),
                    ],
                CreateResource(url='%(url)s'))
            ]
        self.mass_changer = MassChangerNamedPackages(self.ckanclient,
                                        instructions,
                                        dry_run=self.dry_run,
                                        force=self.force)
        self.mass_changer.pkg_name_list = pkg_name_list
        self.mass_changer.run()
        
class TransferUrlMatcher(PackageMatcher):
    def match(self, pkg):
        if not (pkg['url'] or '').strip():
            log.warn('Ignoring package with no URL: %r', pkg['name'])
            return False
##        if not pkg['url'].lower().endswith('.pdf'):
##            log.warn('Ignoring package URL not ending in ".PDF": %r %r',
##                     pkg['name'], pkg['url'])
##            return False
        if pkg['resources']:
            log.warn('Ignoring package with resources already: %r', pkg['name'])
            return False        
        return True



from nose.tools import assert_equal

import ckan.model as model
import ckan.forms
from ckan.lib.create_test_data import CreateTestData
from ckan.tests import *
from ckan.tests.pylons_controller import PylonsTestCase
from ckan.tests.html_check import HtmlCheckMethods

from ckanext.dgu.forms.package_gov3 import get_gov3_fieldset
from ckanext.dgu.tests import *

#from pylons import config

def _get_blank_param_dict(pkg=None, fs=None):
    return ckan.forms.get_package_dict(pkg, blank=True, fs=fs)

def get_fieldset(**kwargs):
    if not kwargs.has_key('user_editable_groups'):
        kwargs['user_editable_groups'] = []
    return get_gov3_fieldset(**kwargs)


class TestFieldset(PylonsTestCase, HtmlCheckMethods):
    @classmethod
    def setup_class(self):
        self.fixtures = Gov3Fixtures()
        self.fixtures.create()

    @classmethod
    def teardown_class(self):
        self.fixtures.delete()

    def test_0_new_fields(self):
        fs = get_fieldset()
        fs = fs.bind(session=model.Session)
        out = fs.render()
        assert out
        assert 'Title' in out, out
        assert 'Identifier' in out, out
        assert 'Mandate' in out, out
        assert 'Revision' not in out, out
        assert 'Extras' not in out
        # default for license
        print out
        self.check_tag(out, 'option', 'value="uk-ogl"', 'selected="selected"')

    def test_0_edit_fields(self):
        fs = get_fieldset()
        pkg = model.Package.by_name(u'private-fostering-england-2009')
        fs = fs.bind(pkg)
        out = fs.render()
        assert out
        # check the right fields are rendered
        assert 'Title' in out, out
        assert 'Identifier' in out, out
        assert 'Mandate' in out, out
        assert 'Revision' not in out, out
        assert 'Extras' not in out

    def test_1_field_values(self):
        fs = get_fieldset()
        pkg = model.Package.by_name(u'private-fostering-england-2009')
        fs = fs.bind(pkg)
        out = fs.render()
        assert out
        expected_values = [
            (fs.title, 'Private Fostering'),
            (fs.date_released, '30/7/2009'),
            (fs.date_updated, '12:30 30/7/2009'),
            (fs.date_update_future, '1/7/2009'),
            (fs.update_frequency, 'annual'),
            (fs.geographic_granularity, 'regional'),
            (fs.geographic_coverage, None, 'England'),
            (fs.temporal_granularity, 'year'),
            (fs.temporal_coverage, None, '6/2008 - 6/2009'),
            (fs.national_statistic, 'True', 'yes'),
            (fs.precision, 'Numbers to nearest 10, percentage to nearest whole number'),
            (fs.url, 'http://www.dcsf.gov.uk/rsgateway/DB/SFR/s000859/index.shtml'),
            (fs.taxonomy_url, '', ''),
            (fs.department, 'Department for Children, Schools and Families'),
            (fs.agency, '', ''),
            (fs.author, 'DCSF Data Services Group'),
            (fs.author_email, 'statistics@dcsf.gsi.gov.uk'),
            (fs.maintainer, '', ''),
            (fs.maintainer_email, '', ''),
            (fs.license_id, u'OKD Compliant::UK Open Government Licence (OGL)'),
            ]
        for vals in expected_values:
            if len(vals) == 2:
                field, expected_render_str = vals
                expected_render_readonly_str = vals[1]
            else:
                field, expected_render_str, expected_render_readonly_str = vals
            if isinstance(field.renderer, ckan.forms.common.SuggestedTextExtraField.SelectRenderer):
                if expected_render_str.startswith('other='):
                    expected_render_str = 'other" type="text" value="' + expected_render_str.strip('other=')
                    expected_render_readonly_str = expected_render_readonly_str.strip('other=')
                else:
                    # multiple choice must have the particular one selected
                    expected_render_str = 'selected" value="' + expected_render_str
            render = field.render()
            render_readonly = field.render_readonly()
            if expected_render_str == '':
                assert 'value=""' in render or 'value' not in render, \
                   'Expected a blank value in render of field %s but got \'%s\'' % \
                   (field.name, render)
            elif expected_render_str:
                assert expected_render_str in render, \
                       'Expected \'%s\' in render of field %s but got \'%s\'' % \
                       (expected_render_str, field.name, render)
            assert expected_render_readonly_str in render_readonly, \
                   'Expected \'%s\' in render_readonly of field %s but got \'%s\'' % \
                   (expected_render_readonly_str, field.name, render_readonly)
        self.check_tag(fs.geographic_coverage.render(), 'geographic_coverage-england', 'value="True"')
        self.check_tag(fs.temporal_coverage.render(), 'temporal_coverage-from', 'value="12:30 24/6/2008"')
        self.check_tag(fs.temporal_coverage.render(), 'temporal_coverage-to', 'value="6/2009"')

    def test_2_field_department_selected(self):
        fs = get_fieldset()
        pkg = model.Package.by_name(u'private-fostering-england-2009')
        fs = fs.bind(pkg)

        dept = fs.department.render()
        assert '<select' in dept, dept
        self.check_tag(dept, 'option', 'value="Department for Children, Schools and Families"', 'selected')
        assert 'option value="other">' in dept, dept
        assert 'Other:' in dept, dept
        assert 'value=""' in dept, dept

        dept_readonly = fs.department.render_readonly()
        assert 'Department for Children, Schools and Families' in dept_readonly, dept_readonly
        assert 'Other' not in dept_readonly, dept_readonly

    def test_2_field_department_none(self):
        # Create package
        model.repo.new_revision()
        pkg = model.Package(name=u'test3')
        model.Session.add(pkg)
        model.repo.commit_and_remove()
        CreateTestData.flag_for_deletion(pkg_names=[u'test3'])

        pkg = model.Package.by_name(u'test3')
        fs = get_fieldset()
        fs = fs.bind(pkg)
        out = fs.render()
        assert out
        dept = fs.department.render()
        dept_readonly = fs.department.render_readonly()
        assert '<select' in dept, dept
        assert '<option selected="selected" value=""></option>' in dept, dept
        assert '<p></p>' == dept_readonly, dept_readonly

    def test_2_field_department_other(self):
        # Create package
        model.repo.new_revision()
        pkg = model.Package(name=u'test2')
        model.Session.add(pkg)
        pkg.extras = {u'department':u'Not on the list'}
        model.repo.commit_and_remove()
        CreateTestData.flag_for_deletion(pkg_names=[u'test2'])

        pkg = model.Package.by_name(u'test2')
        fs = get_fieldset()
        fs = fs.bind(pkg)
        out = fs.render()
        assert out
        dept = fs.department.render()
        dept_readonly = fs.department.render_readonly()
        assert '<select' in dept, dept
        self.check_tag(dept, 'option', 'value="Department for Children, Schools and Families"', '!selected')
        self.check_tag(dept, 'option', 'value="other"', 'selected')
        assert 'Other:' in dept, dept
        assert 'value="Not on the list"' in dept, dept
        assert '<p>Not on the list</p>' == dept_readonly, dept_readonly

    def test_3_restrict(self):
        fs = get_fieldset(restrict=1)
        restricted_fields = ('name', 'department', 'national_statistic')
        for field_name in restricted_fields:
            assert getattr(fs, field_name)._readonly, getattr(fs, field_name)
        
    def test_4_sync_new(self):
        newtagname = 'newtagname'
        indict = _get_blank_param_dict(fs=get_fieldset())
        prefix = 'Package--'
        indict[prefix + 'name'] = u'testname'
        indict[prefix + 'title'] = u'testtitle'
        indict[prefix + 'notes'] = u'some new notes'
        indict[prefix + 'tags'] = u'russian tolstoy, ' + newtagname,
        indict[prefix + 'license_id'] = u'gpl-3.0'
        indict[prefix + 'date_released'] = u'27/11/2008'
        indict[prefix + 'date_updated'] = u'1/4/2008'
        indict[prefix + 'date_update_future'] = u'1/7/2010'
        indict[prefix + 'geographic_granularity'] = u'regional'
        indict[prefix + 'geographic_coverage-england'] = u'True'
        indict[prefix + 'geographic_coverage-wales'] = u'True'
        indict[prefix + 'temporal_granularity'] = u'year'
        indict[prefix + 'temporal_coverage-from'] = u'6/2008'
        indict[prefix + 'temporal_coverage-to'] = u'6/2009'
        indict[prefix + 'national_statistic'] = u'True'
        indict[prefix + 'precision'] = u'Nearest 1000'
        indict[prefix + 'taxonomy_url'] = u'http:/somewhere/about.html'
        indict[prefix + 'department'] = u'testdept'
        indict[prefix + 'agency'] = u'Quango 1'
        indict[prefix + 'resources-0-url'] = u'http:/1'
        indict[prefix + 'resources-0-format'] = u'xml'
        indict[prefix + 'resources-0-description'] = u'test desc'
        fs = get_fieldset().bind(model.Package, data=indict, session=model.Session)
        CreateTestData.flag_for_deletion(pkg_names=[u'testname'],
                                         tag_names=[u'russian',
                                                    u'tolstoy'],)
        
        model.repo.new_revision()
        assert fs.validate()
        fs.sync()
        model.repo.commit_and_remove()

        outpkg = model.Package.by_name(u'testname')
        assert outpkg.title == indict[prefix + 'title']
        assert outpkg.notes == indict[prefix + 'notes']

        # test tags
        taglist = [ tag.name for tag in outpkg.tags ]
        assert u'russian' in taglist, taglist
        assert u'tolstoy' in taglist, taglist
        assert newtagname in taglist

        # test licenses
        assert outpkg.license_id, outpkg
        assert outpkg.license, outpkg
        assert_equal(indict[prefix + 'license_id'], outpkg.license.id)

        # test resources
        assert len(outpkg.resources) == 1, outpkg.resources
        res = outpkg.resources[0]
        assert res.url == u'http:/1', res.url
        assert res.description == u'test desc', res.description
        assert res.format == u'xml', res.format

        # test gov fields
        extra_keys = outpkg.extras.keys()
        reqd_extras = {
            'date_released':'2008-11-27',
            'date_updated':'2008-04-01',
            'date_update_future':u'2010-07-01',
            'geographic_granularity':indict[prefix + 'geographic_granularity'],
            'geographic_coverage':'101000: England, Wales',
            'temporal_granularity':indict[prefix + 'temporal_granularity'],
            'temporal_coverage-from':'2008-06',
            'temporal_coverage-to':'2009-06',
            'national_statistic':'yes',
            'precision':indict[prefix + 'precision'],
            'taxonomy_url':indict[prefix + 'taxonomy_url'],
            'department':indict[prefix + 'department'],
            'agency':indict[prefix + 'agency'],
            }
        for reqd_extra_key, reqd_extra_value in reqd_extras.items():
            assert reqd_extra_key in extra_keys, 'Key "%s" not found in extras %r' % (reqd_extra_key, extra_keys)
            assert outpkg.extras[reqd_extra_key] == reqd_extra_value, \
                 'Extra \'%s\' should equal \'%s\' but equals \'%s\'' % \
                 (reqd_extra_key, reqd_extra_value,
                  outpkg.extras[reqd_extra_key])

    def test_5_sync_update(self):
        # create initial package
        init_data = [{
            'name':'test_sync',
            'title':'test_title',
            'extras':{
              'external_reference':'ref123',
              'date_released':'2008-11-28',
              'date_updated':'2008-04-01',
              'date_update_future':'1/7/2009',
              'geographic_granularity':'testgran',
              'geographic_coverage':'111000: England, Scotland, Wales',
              'temporal_granularity':'testtempgran',
              'temporal_coverage-from':'2007-01-08',
              'temporal_coverage-to':'2007-01-09',
              'national_statistic':'yes',
              'precision':'testprec',
              'taxonomy_url':'testtaxurl',
              'department':'dosac',
              'agency':'testagency',
              },
            }]
        CreateTestData.create_arbitrary(init_data)
        pkg = model.Package.by_name(u'test_sync')
        assert pkg

        # edit it with form parameters
        indict = _get_blank_param_dict(pkg=pkg, fs=get_fieldset())
        prefix = 'Package-%s-' % pkg.id
        indict[prefix + 'name'] = u'testname2'
        indict[prefix + 'notes'] = u'some new notes'
        indict[prefix + 'tags'] = u'russian, tolstoy',
        indict[prefix + 'license_id'] = u'gpl-3.0'
        indict[prefix + 'date_released'] = u'27/11/2008'
        indict[prefix + 'date_updated'] = u'1/4/2008'
        indict[prefix + 'date_update_future'] = u'1/8/2010'
        indict[prefix + 'geographic_granularity'] = u'regional'
        indict[prefix + 'geographic_coverage-england'] = u'True'
        indict[prefix + 'geographic_coverage-wales'] = u'True'
        indict[prefix + 'temporal_granularity'] = u'year'
        indict[prefix + 'temporal_coverage-from'] = u'6/2008'
        indict[prefix + 'temporal_coverage-to'] = u'6/2009'
        indict[prefix + 'national_statistic'] = u'True'
        indict[prefix + 'precision'] = u'Nearest 1000'
        indict[prefix + 'taxonomy_url'] = u'http:/somewhere/about.html'
        indict[prefix + 'department'] = u'testdept'
        indict[prefix + 'agency'] = u'Quango 1'
        indict[prefix + 'resources-0-url'] = u'http:/1'
        indict[prefix + 'resources-0-format'] = u'xml'
        indict[prefix + 'resources-0-description'] = u'test desc'
        fs = get_fieldset().bind(pkg, data=indict)
        CreateTestData.flag_for_deletion(u'testname2')
        
        model.repo.new_revision()
        fs.sync()
        model.repo.commit_and_remove()

        outpkg = model.Package.by_name(u'testname2')
        assert outpkg.notes == indict[prefix + 'notes']

        # test tags
        taglist = [ tag.name for tag in outpkg.tags ]
        assert u'russian' in taglist, taglist
        assert u'tolstoy' in taglist, taglist

        # test licenses
        assert outpkg.license
        assert indict[prefix + 'license_id'] == outpkg.license.id, outpkg.license.id

        # test resources
        assert len(outpkg.resources) == 1, outpkg.resources
        res = outpkg.resources[0]
        assert res.url == u'http:/1', res.url
        assert res.description == u'test desc', res.description
        assert res.format == u'xml', res.format

        # test gov fields
        extra_keys = outpkg.extras.keys()
        reqd_extras = {
            'date_released':'2008-11-27',
            'date_updated':'2008-04-01',
            'date_update_future':'2010-08-01',
            'geographic_granularity':indict[prefix + 'geographic_granularity'],
            'geographic_coverage':'101000: England, Wales',
            'temporal_granularity':indict[prefix + 'temporal_granularity'],
            'temporal_coverage-from':'2008-06',
            'temporal_coverage-to':'2009-06',
            'national_statistic':'yes',
            'precision':indict[prefix + 'precision'],
            'taxonomy_url':indict[prefix + 'taxonomy_url'],
            'department':indict[prefix + 'department'],
            'agency':indict[prefix + 'agency'],            
            }
        for reqd_extra_key, reqd_extra_value in reqd_extras.items():
            assert reqd_extra_key in extra_keys, 'Key "%s" not found in extras %r' % (reqd_extra_key, extra_keys)
            assert outpkg.extras[reqd_extra_key] == reqd_extra_value, \
                 'Extra %s should equal %s but equals %s' % \
                 (reqd_extra_key, reqd_extra_value,
                  outpkg.extras[reqd_extra_key])

    def test_6_sync_update_restrict(self):
        # create initial package
        pkg_name = u'test_sync_restrict'
        init_data = [{
            'name':pkg_name,
            'title':'test_title',
            'extras':{
              'notes':'Original notes',
              'national_statistic':'yes',
              'department':'dosac',
              },
            }]
        CreateTestData.create_arbitrary(init_data)
        pkg = model.Package.by_name(pkg_name)
        assert pkg

        # edit it with form parameters
        indict = _get_blank_param_dict(pkg=pkg, fs=get_fieldset(restrict=1))
        prefix = 'Package-%s-' % pkg.id
        indict[prefix + 'notes'] = u'some new notes'
        # try changing restricted params anyway
        new_name = u'testname3' 
        indict[prefix + 'name'] = new_name
        indict[prefix + 'department'] = u'testdept'
        # don't supply national_statistic param at all
        fs = get_fieldset(restrict=1).bind(pkg, data=indict)
        CreateTestData.flag_for_deletion(new_name)
        
        model.repo.new_revision()
        fs.sync()
        model.repo.commit_and_remove()

        assert not model.Package.by_name(new_name) # unchanged
        outpkg = model.Package.by_name(pkg_name) # unchanged
        assert outpkg
        # test sync worked
        assert outpkg.notes == indict[prefix + 'notes']

        # test gov fields
        extra_keys = outpkg.extras.keys()
        reqd_extras = {
            'national_statistic':'yes', # unchanged
            'department':init_data[0]['extras']['department'], # unchanged
            }
        for reqd_extra_key, reqd_extra_value in reqd_extras.items():
            assert reqd_extra_key in extra_keys, 'Key "%s" not found in extras %r' % (reqd_extra_key, extra_keys)
            assert outpkg.extras[reqd_extra_key] == reqd_extra_value, \
                 'Extra %s should equal %s but equals %s' % \
                 (reqd_extra_key, reqd_extra_value,
                  outpkg.extras[reqd_extra_key])

    def test_7_validate_bad_date(self):
        # bad dates must be picked up in validation
        indict = _get_blank_param_dict(fs=get_fieldset())
        prefix = 'Package--'
        indict[prefix + 'name'] = u'testname3'
        indict[prefix + 'title'] = u'Test'
        indict[prefix + 'department'] = u'abc'
        indict[prefix + 'notes'] = u'abcd'
        indict[prefix + 'license_id'] = u'abcde'
        indict[prefix + 'date_released'] = u'27/11/2008'
        fs = get_fieldset().bind(model.Package, data=indict, session=model.Session)

        validation = fs.validate()
        assert validation, validation

        indict[prefix + 'date_released'] = u'27/11/0208'
        fs = get_fieldset().bind(model.Package, data=indict, session=model.Session)
        validation = fs.validate()
        assert not validation

        # make sure it syncs without exception (this is req'd for a preview)
        model.repo.new_revision()
        fs.sync()
        model.repo.commit_and_remove()

    def test_8_geo_coverage(self):
        # create initial package
        pkg_name = u'test_coverage'
        init_data = [{
            'name':pkg_name,
            'title':'test_title',
            'extras':{
              'geographic_coverage':'001000: England, Scotland, Wales',
              },
            }]
        CreateTestData.create_arbitrary(init_data)
        pkg = model.Package.by_name(pkg_name)
        assert pkg

        # edit it with form parameters
        fs = get_fieldset()
        indict = ckan.forms.get_package_dict(pkg, fs=fs)
        prefix = 'Package-%s-' % pkg.id
        indict[prefix + 'geographic_coverage-england'] = u'True'
        indict[prefix + 'geographic_coverage-wales'] = u'True'
        indict[prefix + 'geographic_coverage-scotland'] = u'True'
        indict[prefix + 'geographic_coverage-global'] = u'True'
        fs = fs.bind(pkg, data=indict)
        
        model.repo.new_revision()
        fs.sync()
        model.repo.commit_and_remove()

        outpkg = model.Package.by_name(pkg_name)
        self.assert_equal(outpkg.extras['geographic_coverage'], '111001: Global, Great Britain (England, Scotland, Wales)')




import xml.sax
import re
import os
import glob
from ckanext.dgu import schema
from swiss import date

guid_prefix = 'http://www.statistics.gov.uk/'

log = __import__("logging").getLogger(__name__)

class OnsImporter(object):
#NB This should derive from ckan/lib/importer:PackageImporter
    def __init__(self, filepath):
        self._filepath = filepath
        self._current_filename = os.path.basename(self._filepath)
        self._item_count = 0
        self._new_package_count = 0
        self._crown_license_id = u'uk-ogl'

    def pkg_dict(self):
        for item in OnsDataRecords(self._filepath):
            yield self.record_2_package(item)

    def record_2_package(self, item):
        assert isinstance(item, dict)

        # process item
        title, release = self._split_title(item['title'])
        munged_title = schema.name_munge(title)
        department, agency = self._source_to_department(item['hub:source-agency'])

        # Resources
        guid = item['guid'] or None
        if guid:
            if not guid.startswith(guid_prefix):
                raise RowParseError('GUID did not start with prefix %r: %r' % (guid_prefix, guid))
            guid = guid[len(guid_prefix):]
            if 'http' in guid: 
                raise RowParseError('GUID de-prefixed should not have \'http\' in it still: %r' % (guid))
        existing_resource = None
        download_url = item.get('link', None)
        descriptors = []
        if release:
            descriptors.append(release)
        if guid:
            descriptors.append(guid)
        description = ' | '.join(descriptors)

        notes_list = []
        if item['description']:
            notes_list.append(item['description'])
        for column, name in [('hub:source-agency', 'Source agency'),
                             ('hub:designation', 'Designation'),
                             ('hub:language', 'Language'),
                             ('hub:altTitle', 'Alternative title'),
                       ]:
            if item[column]:
                notes_list.append('%s: %s' % (name, item[column]))
        notes = '\n\n'.join(notes_list)

        extras = {
            'geographic_coverage': u'',
            'external_reference': u'',
            'temporal_granularity': u'',
            'date_updated': u'',
            'agency': u'',
            'precision': u'',
            'geographical_granularity': u'',
            'temporal_coverage_from': u'',
            'temporal_coverage_to': u'',
            'national_statistic': u'',
            'department': u'',
            'update_frequency': u'',
            'date_released': u'',
            'categories': u'',
            'series':u'',
            }
        date_released = u''
        if item['pubDate']:
            date_released = date.parse(item["pubDate"])
            if date_released.qualifier:
                log.warn('Could not read format of publication (release) date: %r' % 
                         item["pubDate"])
        extras['date_released'] = date_released.isoformat()
        extras['department'] = department or u''
        extras['agency'] = agency or u''
        extras['categories'] = item['hub:theme']
        extras['geographic_coverage'] = self._parse_geographic_coverage(item['hub:coverage'])
        extras['national_statistic'] = 'yes' if item['hub:designation'] == 'National Statistics' or item['hub:designation'] == 'National Statistics' else 'no'
        extras['geographical_granularity'] = item['hub:geographic-breakdown']
        extras['external_reference'] = u'ONSHUB'
        extras['series'] = title if release else u''
        for update_frequency_suggestion in schema.update_frequency_options:
            item_info = ('%s %s' % (item['title'], item['description'])).lower()
            if update_frequency_suggestion in item_info:
                extras['update_frequency'] = update_frequency_suggestion
            elif update_frequency_suggestion.endswith('ly'):
                if update_frequency_suggestion.rstrip('ly') in item_info:
                    extras['update_frequency'] = update_frequency_suggestion
        extras['import_source'] = 'ONS-%s' % self._current_filename 

        author = extras['department'] if extras['department'] else None
        resources = [{
                'url': download_url,
                'description': description
                }]

        # update package
        pkg_dict = {
            'name': munged_title,
            'title': title,
            'version': None,
            'url': None,
            'author': author,
            'author_email': None,
            'maintainer': None,
            'maintainer_email': None,
            'notes': notes,
            'license_id': self._crown_license_id,
            'tags': [], # post-filled
            'groups': ['ukgov'],
            'resources': resources,
            'extras': extras,
            }

        tags = schema.TagSuggester.suggest_tags(pkg_dict)
        for keyword in item['hub:ipsv'].split(';') + \
                item['hub:keywords'].split(';') + \
                item['hub:nscl'].split(';'):
            tag = schema.tag_munge(keyword)
            if tag and len(tag) > 1:
                tags.add(tag)
        tags = list(tags)
        tags.sort()
        pkg_dict['tags'] = tags

        return pkg_dict

    @staticmethod
    def _parse_geographic_coverage(coverage_str):
        geo_coverage_type = schema.GeoCoverageType.get_instance()
        coverage_str = coverage_str.replace('International', 'Global')
        geographic_coverage_db = geo_coverage_type.str_to_db(coverage_str)
        return geographic_coverage_db

    @staticmethod
    def _source_to_department(source):
        dept_given = schema.expand_abbreviations(source)
        department = None
        agency = None

        # special cases
        if '(Northern Ireland)' in dept_given or dept_given == 'Office of the First and Deputy First Minister':
            department = u'Northern Ireland Executive'
        if dept_given == 'Office for National Statistics':
            department = 'UK Statistics Authority'
            agency = dept_given
        if dept_given == 'Education':
            department = 'Department for Education'

        # search for department
        if not department:
            for dept in schema.government_depts:
                if dept_given in dept or dept_given.replace('Service', 'Services') in dept or dept_given.replace('Dept', 'Department') in dept:
                    department = unicode(dept)
                
        if department:
            assert department in schema.government_depts, department
        else:
            if dept_given and dept_given not in ['Office for National Statistics', 'Health Protection Agency', 'Information Centre for Health and Social Care', 'General Register Office for Scotland', 'Northern Ireland Statistics and Research Agency', 'National Health Service in Scotland', 'National Treatment Agency', 'Police Service of Northern Ireland (PSNI)', 'Child Maintenance and Enforcement Commission', 'Health and Safety Executive', 'NHS National Services Scotland', 'ISD Scotland (part of NHS National Services Scotland)', 'Passenger Focus', 'Office of the First and Deputy First Minister', 'Office of Qualifications and Examinations Regulation']:
                log.warn('Double check this is not a gvt department source: %s' % dept_given)
            agency = dept_given
        return department, agency

    @classmethod
    def _split_title(cls, xml_title):
        if not hasattr(cls, 'title_re'):
            cls.title_re = re.compile(r'(.*?)\s-\s(.*)')
        match = cls.title_re.match(xml_title)
        if not match:
            'Warning: Could not split title: %s' % xml_title
            return (xml_title, None)
        return [x for x in match.groups() if x]

class OnsDataRecords(object):
    def __init__(self, xml_filepath):
        self._xml_filepath = xml_filepath

    def __iter__(self):
        ons_xml = OnsXml()
        xml.sax.parse(self._xml_filepath, ons_xml)
        for record in ons_xml.items:
            yield record
    

class OnsXml(xml.sax.handler.ContentHandler):
    def startDocument(self):
        self._level = 0
        self._item_dict = {}
        self.items = []
        
    def startElement(self, name, attrs):
        self._level += 1
        if self._level == 1:
            if name == 'rss':
                pass
            else:
                print 'Warning: Not expecting element %s at level %i' % (name, self._level)
        elif self._level == 2:
            if name == 'channel':
                pass
            else:
                print 'Warning: Not expecting element %s at level %i' % (name, self._level)
        elif self._level == 3:
            if name == 'item':
                assert not self._item_dict
            elif name in ('title', 'link', 'description', 'language', 'pubDate', 'atom:link'):
                pass
        elif self._level == 4:
            assert name in ('title', 'link', 'description', 'pubDate', 'guid',
                            'hub:source-agency', 'hub:theme', 'hub:coverage',
                            'hub:designation', 'hub:geographic-breakdown',
                            'hub:ipsv', 'hub:keywords', 'hub:altTitle',
                            'hub:language',
                            'hub:nscl'), name
            self._item_element = name
            self._item_data = u''

    def characters(self, chrs):
        if self._level == 4:
            self._item_data += chrs

    def endElement(self, name):
        if self._level == 3:
            if self._item_dict:
                self.items.append(self._item_dict)
            self._item_dict = {}
        elif self._level == 4:
            self._item_dict[self._item_element] = self._item_data
            self._item_element = self._item_data = None
        self._level -= 1
